/*----------------------------------------------------------------
 * CLA.H - CSL basic definitions
 * Copyright (C) 198?-199? TopSpeed Software Corporation
 * All rights reserved
 *----------------------------------------------------------------*/
#ifndef _CLA_INCL
#define _CLA_INCL
#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif
#pragma save,name(prefix=>"")

/*-Macros-------------------------------------------------------------------*/
#define forever for(;;)
#define FALSE      0
#define TRUE       1
#define ERR        -1
#define LERR       0xffffffffL
#define OK  0
#define STRMAX     255
#define _FIX_DATA  data(seg_name=>CLAFIXED)

/*--------------------------------------------------------------------------*/

/**/
/* Declaration Statements                                                   */
/**/

#include <clacomp.hpp>

/*-------------------------------------------------------------------
 * Picture parsing stuff
 *-------------------------------------------------------------------*/
#include <clapic.hpp>

extern StrDesc Cla$FILESTRDESC;

/**/
/* Control Statements                                                       */
/**/

void Cla$RUN(/* string, */ int=0, int=0);
int  Cla$CALL(/* string, string */ void);
void Cla$CHAIN(/* string */ void);
long Cla$RUNCODE(void);

void Cla$SHUTDOWN(void (*func)(void));
void Cla$code(void);

long Cla$STATUSfile( Cla_FCBBLK *fcb );

void /*string*/ Cla$ERRORFILE(void);

/**/
/* Memory Table Processing And Misc Functions                               */
/**/

#define        GETMEM(a)        calloc(1,a)

#define Cla$_OS_addptr32(mem,size) ((char *)mem+size)

void Cla$ADDqueue(QUEUECB *queue);
void Cla$ADDqueueptr(QUEUECB *queue, long pointer);
void Cla$ADDqueuekey(QUEUECB *queue, int fld, ... );
void Cla$ADDqueueskey(QUEUECB *queue); // String now on stack
void Cla$DELETEqueue(QUEUECB *queue);
void Cla$DISPOSEqueue(QUEUECB *queue);
void Cla$FREEqueue(QUEUECB *queue);
void Cla$GETqueueptr(QUEUECB *queue, long pointer);
void Cla$GETqueuekey(QUEUECB *queue, int fld, ... );
void Cla$GETqueueskey(QUEUECB *queue);
QUEUECB *Cla$NEWqueue(void *typeblock);
void Cla$PUTqueue(QUEUECB *queue);
void Cla$PUTqueuekey(QUEUECB *queue, int fld, ...);
void Cla$PUTqueueskey(QUEUECB *queue);
void Cla$SORTqueuekey(QUEUECB *queue, int fld, ...);
void Cla$SORTqueueskey(QUEUECB *queue);
long Cla$RECORDSqueue(QUEUECB *queue);
long Cla$POINTERqueue(QUEUECB *queue);
long Cla$MODIFIEDqueue(QUEUECB *queue);
long Cla$NUMCHANGESqueue(QUEUECB *queue);
unsigned Cla$NUMFIELDSqueue(QUEUECB *queue);

/**/
/* Mathematical Functions                                                   */
/**/

/*@@   Cla$ABS()      inline code */
/*@@   Cla$INRANGE()  inline code */
double Cla$INT(double x);
double Cla$LOGE(double x);
double Cla$LOG10(double x);
long   Cla$RANDOM(long low,long high);
double Cla$ROUND(double x,double order);
double Cla$SQRT(double x);

/**/
/* Trigonometric Functions (in radians)                                     */
/**/

double Cla$ACOS(double radians);
double Cla$ASIN(double radians);
double Cla$ATAN(double radians);
double Cla$COS(double radians);
double Cla$SIN(double radians);
double Cla$TAN(double radians);

/**/
/* Bit Manipulation Functions                                               */
/**/
/* @@  Cla$BAND();         inline code */
/* @@  Cla$BOR();          inline code */
long   Cla$BSHIFT(long x,int count);
/* @@  Cla$BXOR();         inline code */

/**/
/* Miscellaneous compiler helper functions                                  */
/**/
double Cla$longpower(double x,long y);
double Cla$realpower(double x,double y);
double Cla$modulus(double x,double y);
int    Cla$realdistinct(double x, double y);
int    Cla$srealdistinct(float x, float y);
double Cla$divide(double x, double y);
long   Cla$lroundd(double x);
unsigned long Cla$uroundd(double x);
long          Cla$lroundf(float x);
unsigned long Cla$uroundf(float x);

/**/
/* String Functions                                                         */
/**/

    StrDesc     Cla$ALL(char *string,unsigned slen,unsigned dlen);
    StrDesc     Cla$CENTER(char *string,unsigned slen,unsigned dlen);
    StrDesc     Cla$CLIP(char *string,unsigned len);

    unsigned    Cla$INSTRING(char *sub,unsigned sublen,char *string,
                                  unsigned slen,unsigned step,unsigned start);

    unsigned    Cla$_instring(char *sub,unsigned sublen,char *string,
                                  unsigned slen,unsigned step,unsigned start);

    StrDesc     Cla$LEFT(char *string,unsigned slen,unsigned dlen);
    StrDesc     Cla$LOWER(char *string,unsigned slen);
    unsigned    Cla$NUMERIC(char *string,unsigned slen);
    StrDesc     Cla$RIGHT(char *string,unsigned slen,unsigned dlen);
    StrDesc     Cla$SUB(char *string,unsigned slen,int pos,unsigned len);
    StrDesc     Cla$CAPITAL(char *string,unsigned slen);
    StrDesc     Cla$UPPER(char *string,unsigned slen);

unsigned      Cla$clipstr(char *string);                        /* clastr   */
unsigned      Cla$termstr(char chr,void *string);               /* clastr   */
char *        Cla$fillstr(char chr,void *string,unsigned len);  /* clastr   */
char *        Cla$padstr(void *string,unsigned len);            /* clastr   */
int           Cla$emptystr(char *string, int len);              /* clastr   */
unsigned      Cla$lftjst(char *string);                         /* clastr   */
unsigned      Cla$rgtjst(char *string);                         /* clastr   */
void          Cla$ltabexp(unsigned tabcnt,char *ibuf,           /* clatbexp */
                          unsigned ilen,char *obuf,unsigned max);
unsigned      Cla$censtr(void *string);                         /* clastr   */
char          *Cla$nextword(char *loc, char *word);
char          *Cla$nextwordtok(char *loc, char *word, char *toks);
void          Cla$blankstr(char *string,unsigned slen);

/**/
/* Data Type Conversion/Handling Functions                                  */
/**/

    void        Cla$freestr(StrDescP s);
    StrDesc     Cla$loadstr(char *s,unsigned len);
    void        Cla$storestr(char *d, unsigned dl, char *s, unsigned sl);
    void        Cla$storecstr(char *dst,unsigned dlen,void *src,unsigned slen);
    void        Cla$storepstr(char *dst,unsigned dlen,void *src,unsigned slen);
    int         Cla$comparestr(char *s1,unsigned s1l,char *s2,unsigned s2l);
    StrDesc     Cla$concatstr(char *s1,unsigned s1l,char *s2,unsigned s2l);
    unsigned    Cla$cstrlen(char *s);
    long        Cla$strtolong(char *s,unsigned l);
    StrDesc     Cla$longtostr(long l);
    double      Cla$strtoreal(const char *string,unsigned len);
    StrDesc     Cla$realtostr(double v);
    double      Cla$loaddec(void *deciptr,unsigned char declen,unsigned char dplace);
    void        Cla$storedec(void *d,unsigned char declen,unsigned char dplace,double v);
    double      Cla$ploaddec(void *deciptr,unsigned char declen,unsigned char dplace);
    void        Cla$pstoredec(void *d,unsigned char declen,unsigned char dplace,double v);
    long        Cla$pictloadlong(const char *s, unsigned l, const char *ptok);
    double      Cla$pictloadreal(const char *s, unsigned l, const char *ptok);
    void        Cla$pictstorelong(char *s, unsigned l, const char *ptok,long v);
    void        Cla$pictstorereal(char *s, unsigned l, const char *ptok,double v);
    void        Cla$duplicate(char *dst,unsigned dlen,unsigned count);
    void        Cla$storeblock(char *d,char *s,unsigned len);
    StrDesc     Cla$FORMATstr_horiz_scroll(char *s,unsigned l,char *ptok);

void     Cla$pictcount(PicDesc *pic,PicCounts *count);
void     Cla$dec2asc(void *deciptr,unsigned iplace,unsigned dplace,char *obuf);
void     Cla$idec2asc(void *deciptr,unsigned iplace,unsigned dplace,char *obuf);
char     *Cla$deformatpict(const char *s, unsigned l, const char *ptok);
void     Cla$formatpict(char *dest, unsigned l, const char *ptok,int dec,int sign);

StrDesc     Cla$DEFORMAT(const char *string,unsigned slen,const char *pic);
StrDesc     Cla$FORMATstr(const char *string,unsigned slen,const char *pic);
StrDesc     Cla$FORMATlong(long v,const char *pic);
StrDesc     Cla$FORMATreal(double v,const char *pic);


double  Cla$bloadfloat(void *in);
void    Cla$bstorefloat(void *in,double d);
double  Cla$bloadfloat4(void *in);
void    Cla$bstorefloat4(void *in,double d);

/**/
/* Date Functions                                                           */
/**/

    StrDesc	Cla$AGE(long bday,long base);
    long	Cla$DATE(unsigned mo,unsigned dy,unsigned yr);
    long	Cla$DAY(long cdate);
    long	Cla$MONTH(long cdate);
    long	Cla$TODAY(void);
    long	Cla$YEAR(long cdate);
    void	Cla$SETTODAY(long cdate);

    typedef struct {
	unsigned char	dy;
	unsigned char	mo;
	unsigned short  yr;
    } btdate;

    long	Cla$loadbtdate(long date);
    long	Cla$storebtdate(long date);

    int		Cla$valabs(long absday);
    int		Cla$valdate(unsigned month,unsigned day,unsigned year);
    long	Cla$date2abs(struct date *d);
    int		Cla$abs2date(long absday,struct date *d);
    int   Cla$Intellidate(int yr,char years_before = 80);

/**/
/* Time Functions                                                           */
/**/

    long	Cla$CLOCK(void);
    void	Cla$SETCLOCK(long ctime);

    typedef struct {
	char	    hun;
	char	    sec;
	char	    min;
	char	    hour;
    } bttime;

    long	Cla$loadbttime(long time);
    long	Cla$storebttime(long time);
    long	Cla$time2abs(struct time *tm);
    int		Cla$abs2time(long abstim,struct time *tm);

/**/
/* Error Handling Functions                                                 */
/**/

extern unsigned     Cla$DOSerror;
extern int          Cla$ERRCODE;              /* global error number field */
extern char         Cla$ERRSTR[80];           /* global error description  */
extern Cla_FCBBLK   *Cla$FileError;           // pointer to fcb of last file w/error

    void        Cla$seterror(int err);
    void        Cla$pusherrcodes();
    void        Cla$poperrcodes();
    void        Cla$ovrerror(int err,char *msg);
    void        Cla$clearerror(void);
    void        Cla$rterr(int err,void *msg);
    char        *Cla$err_message(unsigned err);
    void        Cla$setfileerror(char *, char *);
    void        Cla$FILEERRORMSG(void);
    void        Cla$FILEERRORCODE(void);
    void        Cla$SetErrorFile(Cla_FCBBLK  *File);

/**/
/* Miscellaneous Statements                                                 */
/**/

typedef struct {
	char		fld_type;
	unsigned 	fld_offset;
	unsigned	fld_len;
	char		*t_desc;
}FLDDESC;

#pragma save,call(reg_param=>(si,ds,cx),reg_saved=>(bx,dx,di,es,st1,st2), inline=>on)
static unsigned Cla$crc16xmod(char *p, unsigned len)=
{
    0x53,                   // PUSH  BX
    0x52,                   // PUSH  DX
    0x57,                   // PUSH  DI
    0xBB,0x00,0x00,         // MOV   BX,0000
    0xBF,0x21,0x10,         // MOV   DI,1021
    0x51,                   // PUSH  CX
    0xAC,                   // LODSB
    0xB9,0x08,0x00,         // MOV   CX,0008
    0x8A,0xE0,              // MOV   AH,AL
    0x80,0xE4,0x80,         // AND   AH,80
    0x8B,0xD3,              // MOV   DX,BX
    0xD1,0xE2,              // SHL   DX,1
    0x80,0xE7,0x80,         // AND   BH,80
    0x32,0xFC,              // XOR   BH,AH
    0x8B,0xDA,              // MOV   BX,DX
    0x74,0x02,              // JZ    01AF
    0x33,0xDF,              // XOR   BX,DI
    0xD0,0xE0,              // SHL   AL,1
    0xE2,0xE8,              // LOOP  019B
    0x59,                   // POP   CX
    0xE2,0xE0,              // LOOP  0196
    0x8B,0xC3,              // MOV   AX,BX
    0x5F,                   // POP   DI
    0x5A,                   // POP   DX
    0x5B,                   // POP   BX
};
#pragma restore

#pragma save,call(reg_param=>(es,di,ax,cx),reg_saved=>(si,ds,st1,st2), inline=>on)
static size_t scn_str(const char *_s, unsigned c, unsigned len )=
{
    0x8B, 0xD9, //               mov   bx,cx
    0x47,        //              inc   di
    0xF3, 0xAE,  //              rep;  scasb
    0x2B, 0xD9,      //          sub   bx,cx
    0x8B, 0xC3,      //          mov   ax,bx
    0x90             //          nop         ;to satisfy compiler/linker bug
};
#pragma restore

    void	Cla$DELAY(unsigned millisecs);
    void	Cla$BEEP(unsigned freq,unsigned dur);
    void	Cla$IN(unsigned port,unsigned *input,unsigned bytes);
    void	Cla$OUT(unsigned port,void *output,unsigned bytes);
    void	Cla$PEEK(unsigned long a,void *r, unsigned len);
    void	Cla$POKE(unsigned long a,void *v, unsigned len);

    void        Cla$DIRECTORY(QUEUECB * tcb, unsigned char attrib );
    void	Cla$SETPATH(void);
    void	Cla$PRINTstr(char *str,unsigned slen);

    unsigned long Cla$crc32(char *buf, unsigned len, unsigned long crc);
    unsigned Cla$crc16(unsigned crc,unsigned len,void *buffer);

    unsigned Cla$pack(char *src, char *dst, unsigned len);
    unsigned Cla$unpack(char *src, char *dst);
    int      Cla$psprintf(char *outs, char *form, char **pblock, char *tblock );

    int     Cla$fullpath(char *fname, char *fullpath, int pathsize );

    int Cla$fnsplit(char *path, char *drv, char *dir, char *name, char *ext );
    char *Cla$fextract(char *path, int drv, int dir, int name, int ext );
    void findclose(struct ffblk *buffer);


  unsigned long Cla$MEMORY(unsigned mode);

/**/char *	Cla$ERROR(char *dest,unsigned dlen);
/**/unsigned	Cla$ERRORCODE(void);
    void /*string*/ Cla$PATH(void);

/**/
/* International Sort                                                       */
/**/

typedef struct {
    unsigned char   collate[256];
    unsigned char   alphacase[256];
    unsigned char   caseflg[256];
    char            mo[12][4];
    char            month[12][13];
    char            am[3];
    char            pm[3];
    char            use;
    char            digraph[16][4];
    char            oemdigraph[16][4];
// beneath are for windows only
    unsigned char   oemcollate[256];
    char            ShortDatePdesc[INTL_MAX_SHORT_DATE_WIDTH*2+4];
    char            LongDatePdesc[INTL_MAX_LONG_DATE_WIDTH*2+4];
    char            ShortTimePdesc[INTL_MAX_SHORT_TIME_WIDTH*2+4];
    char            LongTimePdesc[INTL_MAX_LONG_TIME_WIDTH*2+4];
    char            WinAmPm[2][3];

} ISFinfo;

extern ISFinfo Isfinfo;

void Cla$isfinit(void);
int  Cla$isfisalpha(unsigned char c);
int  Cla$isfisalnum(unsigned char c);
int  Cla$isfislower(unsigned char c);
int  Cla$isfisupper(unsigned char c);
char Cla$isftoupper(unsigned char ch);
char Cla$isftolower(unsigned char ch);
int  Cla$isfcompare(const void *src1,const void *src2,unsigned len);
int  Cla$isfoemcompare(const void *src1,const void *src2,unsigned len);
int  Cla$isfcomparec(unsigned char,unsigned char);
int  Cla$isficompare(const void *src1,const void *src2,unsigned len);
char *Cla$isflower(char *ch,unsigned len);
char *Cla$isfupper(char *ch,unsigned len);
char *Cla$isfconvert(char *ch,unsigned len);
char *Cla$isfoemconvert(char *ch,unsigned len);

/**/
/* Option Arguments                                                         */
/**/

#define TRAILING_BACKSLASH      1
#define WILD_CARDS_IN_TAIL      2
#define NO_PATH_SPECIFICATION   4
#define UNIT_TOO_LONG           8
#define PATH_TOO_LONG           16
#define INVALID_DRIVE_LETTER    32
#define INVALID_CHARACTER       64
#define EXTENSION_TOO_LONG     128
#define MISSPLACED_DRIVE_SPEC  512
#define MISSING_DIRECTORY     1024
#define MULTIPLE_EXTENSIONS   2048

#define	ANYLINE		0
#define ENVIRONMENT	1
#define CONFIGFILE	2
#define	COMMANDLINE	3

char    *Cla$getswitch( char *s, unsigned op );
void     Cla$_setcommand( char *s );
void /* string */ Cla$COMMAND( /* string, */ unsigned );
void     Cla$SETCOMMAND( /* string */ void);
void     Cla$arg_init(void);
void     Cla$arg_close(void);
void     Cla$ctl_update(char *str,char *val);
void     Cla$ctl_store(void);
int      Cla$stdcmp(void *, void *, unsigned);
unsigned Cla$validpathname(char *path);
char    *Cla$progname(char *buf);
int      Cla$write_command(char *flag, char *inifile, int create);

/**/
/* Transaction Logging handlers                                             */
/**/

void Cla$LOGOUT(int seconds, ... );
void Cla$ROLLBACK(void);
void Cla$COMMIT(void);

void Cla$activate_drv(  void *fcb,
                        int acc_mode,
                        int logoutop,
                        int rollbackop,
                        int commitop,
                        int lockop,
                        int unlockop);

void Cla$deactivate_drv(void *fcb);
long Cla$getfilemode(Cla_FCBBLK *fcb);

#ifdef _WIN
void Cla$CloseAll(void);
#endif

#pragma restore
#ifdef __cplusplus
}
#endif

#define _PROT  // for windows
#endif // _CLA_INCL
