/*-----------------07/09/95 09:19-------------------
   This is a re-work of the type descriptor class to deal with the new
 class descriptors. Code currently using clatdesc can switch to this by
 renaming a few classes.
 --------------------------------------------------*/
/*-----------------02-17-94 11:40pm-----------------
 This defines the interface to the type-descriptor management
 routines. They are designed to be semi-efficient.
 The idea is you have a type-descriptor object that you send fields to,
 it then builds up the type-descriptor into a piece of xxxxx using virtual
 functions.
 --------------------------------------------------*/

/*-----------------02-19-94 10:24am-----------------
 Known weaknesses:
   Does not guard against OVERs being bigger than underlying field
 --------------------------------------------------*/
#ifndef inc_clacdesc
#define inc_clacdesc
#if COMPILER
#include "stdarg.h"
#endif
#include "clacomp.hpp"
#ifndef COMPILER
#include "claufo.hpp"
#endif
#define MAX_ID  100
#define MAX_DIM  20

typedef unsigned char byte;

//Internal error
#define TERR_DABSCREWEDUP  0
//Type descriptor corrupt
#define TERR_CORRUPT       1
//Attempt to perform operation on incomplete type descriptor
#define TERR_INCOMPLETE    2
//Mis-matched Start/End groups
#define TERR_BADNEST       3
//Something got bigger than 64K
#define TERR_TOOBIG        4
//Attempt to use unfound field name
#define TERR_NOTFOUND      5
//Attempt to over a smaller or non-existent field
#define TERR_BADOVER       6
//Attempt to over-write existing type descriptor (illegally)
#define TERR_WRITEOVER     7
//Attempt to overshoot with field number
#define TERR_HIGHFIELD     8
//Name greater than MAX_ID
#define TERR_MAXID         9
//Name greater than MAX_DIM
#define TERR_MAXDIM        10
//Must have outer level group
#define TERR_NOGROUP       11


class class_descriptor;
class VarUfo;
class field_descriptor
{
  public:
    field_descriptor() { reset(); };
    field_descriptor( class_descriptor *p ) { parent = p; reset(); };
    void dump();
    void read();
    void reset();
    unsigned nelements(); // Dimensions multiplied out
    char name[MAX_ID];
    char comment[MAX_ID];
    char ndims;
    unsigned dim[MAX_DIM];
    class_descriptor *parent;
    TypeCode type;
    long offset_delta;    // Delta from previous field end
    unsigned size;
    unsigned info;        // Misc info (decimal(,x))
    char picture[MAX_ID]; // Bit OTT but still
    unsigned nfields;     // Fields in group
    unsigned textent;     // Extent of group type descriptor
    unsigned cwm;         // Only used for tracking group offsets
    byte level;           // Nested group level
#ifndef COMPILER
    VarUfo *field2ufo(void *);
#endif
};

/*-----------------02-18-94 01:26pm-----------------
 Field numbers: For these purposes fields are counted as items which could
 have labels. This means (roughly) a line-count without Ends. eg
G GROUP             ! Field 0
L1 SHORT,DIM(4)     ! Field 1
L2 GROUP            ! Field 2
L3 SHORT            ! Field 3
L4 SHORT,OVER(L3)   ! Field 4
   END
L5 SHORT            ! Field 5
 END
 --------------------------------------------------*/
class class_descriptor
{
    friend class field_descriptor;
  public:
    // Called before corresponding AddField or StartGroup
    // For multiple dimensions use major --> minor order
#ifndef COMPILER
    VarUfo *Field2Ufo( unsigned, void *record );
    VarUfo *OldField2Ufo( unsigned, void *record );
#endif
    void AddArray( unsigned );
    void AddField( TypeCode, unsigned size, const char *name = NULL, unsigned info = 0, const char *pic = NULL, const char *comment = NULL );
    void AddSimple( TypeCode, const char *name = NULL, const char *comment = NULL );
    void EndDeclaration( void );
    void EndGroup( void );
    char *FieldName( unsigned ); // Field name from number, must be free'd
    unsigned FieldNum( const char *s ); // Get field number from name
    field_descriptor *FieldDesc( unsigned );
    field_descriptor *OldFieldDesc( unsigned );
    unsigned NFields( void );
    void OverField( unsigned ); // Called before corresponding AddField or StartGroup
    void SetOffset( unsigned ); // Called before AddField etc
    void Print( void );
    void StartGroup(const char *name = NULL);
/*-----------------02-17-94 11:48pm-----------------
 The user of this class must undertake to implement a 'file' for the type
 descriptor to be written into. This will probably be some memory. The
 size required cannot be predicted (hence this interface) but I have assumed
 it will be <64K.
 --------------------------------------------------*/
  protected:
  /*-----------------02-18-94 00:37am-----------------
   inline_names should be set to 1 to indicate BINDABLE. 0 to indicate
   non_bindable
   named_overs indicates that this class should maintain a symbol table of
   the record to allow fields to be indicated by name rather than number.
   --------------------------------------------------*/
    ~class_descriptor( );
    void setup(char inline_names, char named_fields);
    virtual void   error(unsigned) = 0;
    virtual void   free_mem(void *) = 0;
    virtual byte   get_byte( void ) = 0;
    virtual void * get_mem(unsigned l) = 0;
    virtual void   position(unsigned) = 0;
    virtual void   positionr(int) = 0;
    virtual void   put_byte(byte c) = 0;
    virtual void   put_char(char c) = 0;
    virtual void   put_newline( void ) = 0;
    virtual unsigned tell( void ) = 0;
    virtual int    type_size(char c) = 0;
  private:
#pragma save,call(near_call=>on)
    void reset_cache( void );
    void complete( void );
    byte field_position(unsigned,unsigned&); // Get to field, returns depth
    byte field_position(unsigned n)
    {
      unsigned temp;
      return field_position(n,temp);
    }  ; // Get to field, returns depth
    unsigned field_offset(unsigned n)
    {
      unsigned temp;
      field_position(n,temp);
      return temp;
    }
    unsigned get_word( void );
    short get_short( void );
    long get_long( void );
    long get_sized();
    void get_cstring(char *);
    void new_field( void );
    void new_name(const char *);
    void put_chars(const char *s);
    void put_cstring(const char *s);
    void put_number(unsigned);
    void put_word(unsigned);
    void put_sized( long );
    void skip_bytes(unsigned);
#pragma restore
    char **names;
    byte group_nest;
    field_descriptor *fields;
    field_descriptor current_field;
    struct field_info
    {
      unsigned toffset;
      unsigned offset;
      unsigned tdims;
      char     level;
    } *finfo;
    unsigned nfields,curfield;
    byte inames;
    byte done;
    unsigned coffset;
};

/*-----------------02-17-94 11:51pm-----------------
 A suggested 'model' derivation which would work in the library
 --------------------------------------------------*/
#ifndef COMPILER
#include <clwclib.h>

class ClassDesc : public class_descriptor
{
  public:
    // Registers existing typedesc and implicit position 0
    ClassDesc( byte *b, byte no );
    ClassDesc(byte in,byte no);
    ~ClassDesc( )
    {
      if ( freemem )
        delete [] base;
    }
    unsigned DSize( )
    {
      return finger - base;
    }
    void Dump(void *b)
    {
      memcpy(b,base,DSize( ));
    }
    void Over(char *s)
    {
      OverField(FieldNum(s));
    }
  protected:
    virtual void error(unsigned n);
    virtual void free_mem(void *v)
    {
      delete [] v;
    }  ;
    virtual byte get_byte( )
    {
      return *finger++;
    }  ;
    virtual void position(unsigned p)
    {
      finger = base + p;
    }  ;
    virtual int type_size(char c);
    virtual void positionr(int p)
    {
      finger += p;
    }  ;
    virtual void put_byte(byte c)
    {
      *finger++ = c;
    };
    virtual void *get_mem(unsigned l)
    {
      return new char [ l ];
    };
    virtual void put_char(char c);
    virtual void put_newline( );
    virtual unsigned tell( )
    {
      return finger - base;
    }
  private:
    byte *base,*finger;
    byte freemem;
};

/*-----------------02-22-95 03:19pm-----------------
 record_ufos class. A ufo-descriptor manager for a given record

 The idea is you can create a tribe of UFOs for a given record structure and
 can then 'latch' the tribe of UFOs onto any instance of that record that
 comes along.

 Ufos are not created until needed, set_base gets more expensive as more
 fields are used. One may do a 'clear' to empty out the ufo cache if
 set_bases are common and the ufo usage profile has changed significantly

 --------------------------------------------------*/

class record_ufos
{
  private:
    void *current_offset;
    class_descriptor &manage;
    VarUfo **ufos;
  public:
    record_ufos( class_descriptor & t, void *offset);
    ~record_ufos( );
    Ufo *field(unsigned n);
    Ufo *old_field(unsigned n);
    void set_base(void *offset);
    void clear(void);
    unsigned fields();
};

#endif

#endif
