#ifndef _CLACOMP_INCL
#define _CLACOMP_INCL
/*-----------------------------------------------------------------------
 * Clarion runtime library compiler data interfaces
 *
 * All constants and data structures which the compiler generates
 * for use by the library are defined HERE and ONLY HERE!
 *-----------------------------------------------------------------------*/

/*------------------------------------------------------------------
 * queues
 *------------------------------------------------------------------*/

class qgroup;
typedef struct
{
  char            *data;
  char            *tdp;
  qgroup         *extra;
  unsigned        tdsize;
} QUEUECB;

/*------------------------------------------------------------------
 * strings
 *------------------------------------------------------------------*/

typedef struct
{
  char     *str;
  unsigned  len;
} StrDesc;

#ifdef __32BIT__
typedef StrDesc *StrDescP;
#else
 #if defined(__cplusplus)
 typedef StrDesc near *StrDescP;
 #else
 typedef StrDesc <_SS> *StrDescP;
 #endif
#endif

/*------------------------------------------------------------------
 * keys
 *------------------------------------------------------------------*/

typedef struct {     // Key Component Structure
    int     fld_num; // field number in record structure (-ve = descending)
} Cla_KEYCOMP;

typedef struct Cla_FCBBLK_struct Cla_FCBBLK;
typedef struct Cla_VCBBLK_struct Cla_VCBBLK;

typedef struct {                // Key Descriptor Structor
    StrDesc         pname;      // physical key name
    StrDesc         lname;      // label name for key
    void           *rt_kdesc;   // additional runtime key info
    unsigned        key_attr;   // attribute of key
    unsigned char   num_comps;  // number of components for key
    Cla_KEYCOMP     *comps;     // pointer to components
    Cla_FCBBLK      *parent;    // pointer to owning file
} Cla_KEYDESC;

// key_attr flags

#define ClaDUP      0x01    // key allows duplicates
#define ClaOPT      0x02    // suppress creation of keys with null value
#define ClaNOCASE   0x04    // key values are case insensitive
#define ClaAUTOINC  0x08    // 2 or 4 bytes autoincremented key
#define ClaPRIMARY  0x10    // primary key

// The following key type is a clarion key type
#define INDEX       0x4000  // key is a external index file
#define DYNINDEX    0x8000  // key is a dynamic external index file

/*------------------------------------------------------------------
 * Memos
 *------------------------------------------------------------------*/

typedef struct {                  // Memo Descriptor Structure
    StrDesc         memo_name;    // field name of memo
    StrDesc         memo_label;   // label name of memo
    void           *memo_data;    // pointer to memo data area
    void           *rt_mdesc;     // additional runtime memo info
    unsigned char   memo_num;     // runtime memo number
    unsigned char   memo_attr;    // indicates memo or image
    unsigned        memo_size;    // total size of memo
} Cla_MEMODESC;

// memo_attr values

#define MEMO    0x01
#define IMAGE   0x02
#define BLOB    0x04

/*------------------------------------------------------------------
 * Files
 *------------------------------------------------------------------*/

typedef long (*Cla_PIPE_FUNC)(int,Cla_FCBBLK*,va_list);  // pointer to pipe function
typedef long (*Cla_VIEW_FUNC)(int,Cla_VCBBLK*,va_list);  // pointer to view pipe function

struct Cla_VCBBLK_struct
{
  Cla_VIEW_FUNC pipe;
  char * ViewDescriptor;
  void ** k;
  char * filter;
  char *order;
  void ** memo_ptr;
};

struct Cla_FCBBLK_struct {          // File Control Block Structure
    Cla_PIPE_FUNC pipe;             // (compiler) pointer to pipe function
    StrDesc       phyname;          // (compiler) physical filename
    StrDesc       drvstr;           // (compiler) file driver string
    StrDesc       oname;            // (compiler) owner name for file
    unsigned char fil_attr;         // (compiler) file attribute
    unsigned char num_keys;         // (compiler) number of keys in this file
    unsigned char num_memos;        // (compiler) number of memos in this file
    unsigned      rec_len;          // (compiler) record length
    void          *t_desc;          // (compiler) pointer to RECORD type desc.
    Cla_KEYDESC   **k_desc;         // (compiler) pointer to block of KEY point.
    Cla_MEMODESC  **m_desc;         // (compiler) pointer to block of MEMO point.
    void          *rec_buf;         // (compiler) pointer to record buffer
    void          *rblock;          // (runtime)  file drivers runtime block
    unsigned char  threadid;        // (runtime)  original owner thread
    unsigned char  threadflags;     // (runtime)  thread flags
};

// fil_attr flags

#define FILOWN  0x01    // owner
#define FILCRYP 0x02    // records are encrypted
#define FILCREA 0x04    // file can be created
#define FILRECL 0x08    // reclaim unused space
#define FILTHRD 0x10    // file is threaded
#define FILOEM  0x20    // String data translated to OEM character set

/*------------------------------------------------------------------
 * Data type constants
 *------------------------------------------------------------------*/


typedef  unsigned char typkind;
enum
{
   TYP_END,TYP_BYTE,TYP_SHORT,TYP_USHORT,TYP_BDATE,TYP_BTIME,TYP_LONG,
   TYP_ULONG,TYP_FLOAT,TYP_REAL,TYP_DECIMAL,TYP_PDECIMAL,TYP_STACKDECIMAL,
   TYP_BFLOAT4, TYP_BFLOAT8,
   TYP_UFO, TYP_TEMPUFO,
   TYP_STRING = 18,TYP_CSTRING,TYP_PSTRING,TYP_MEMO,
   TYP_GROUP,TYP_CLASS,TYP_DIM,TYP_FIELD,TYP_QUEUE,

   TYP_TEMPSTR,
   TYP_STACKSTR,
   TYP_PICTURE,
   TYP_STDNUM,
   TYP_REFERENCE,
   TYP_FILEREF,
   TYP_KEYREF,
   TYP_QUEUEREF,
   TYP_CLASSREF,
   TYP_WINDOWREF,
   TYP_error
};

#if TARGET32
/*-----------------21-12-94 09:31am-----------------
 This is not quite a dopey as it looks, LONG -is- the most efficient way
 of storing smallish unsigned numbers in Clarion 32 bit.
 --------------------------------------------------*/
#define TYP_UNSIGNED TYP_LONG
#define TYP_SIGNED   TYP_LONG
#else
#define TYP_UNSIGNED TYP_USHORT
#define TYP_SIGNED   TYP_SHORT
#endif

#define IsString(t) ( (t) >= TYP_STRING && (t) <= TYP_PSTRING || (t) >= TYP_TEMPSTR && (t) <= TYP_PICTURE )
#define IsDecimal(t) ( (t) >= TYP_DECIMAL && (t) <= TYP_STACKDECIMAL )
#define IsLong(t) ( (t) <= TYP_LONG )
#ifdef COMPILER
#define IsInteger(t) ( IsLong(t) || bcd.ulong && (t) == TYP_ULONG )
#else
#define IsInteger(t) ( IsLong(t) || (t) == TYP_ULONG )
#endif
#define IsFixed(t) ( IsInteger(t) || IsDecimal(t) )
#define UnderDecimal(t) ( IsFixed(t) || IsString(t) )
#define IsFixed(t) ( IsInteger(t) || IsDecimal(t) )
#define IsFloat(t) ( UnderReal(t) && !IsFixed(t) )
#define IsReal(t) ( (t) >= TYP_FLOAT && (t) <= TYP_BFLOAT8 && !IsDecimal(t) )
#define UnderReal(t) ( (t) <= TYP_BFLOAT8 )
#define IsGroup(t) ( (t) == TYP_CLASS || (t) == TYP_GROUP || (t) == TYP_QUEUE )
#define IsStruct(t) ( IsGroup(t) || (t) == TYP_CLASSREF || (t) == TYP_QUEUEREF )
#define IsTypeRef(t) ( (t) >= TYP_REFERENCE && (t) < TYP_error )

#define TypeCode typkind

#define ClaEND TYP_END
#define ClaBYTE TYP_BYTE
#define ClaSHORT TYP_SHORT
#define ClaUSHORT TYP_USHORT
#define ClaDATE TYP_BDATE
#define ClaTIME TYP_BTIME
#define ClaLONG TYP_LONG
#define ClaULONG TYP_ULONG
#define ClaFLOAT TYP_FLOAT
#define ClaREAL TYP_REAL
#define ClaDECIMAL TYP_DECIMAL
#define ClaPDECIMAL TYP_PDECIMAL
#define ClaIDECIMAL TYP_PDECIMAL
#define ClaSTACKDEC TYP_STACKDEC
#define ClaBFLOAT4 TYP_BFLOAT4
#define ClaBFLOAT8 TYP_BFLOAT8
#define ClaUFO TYP_UFO
#define ClaTEMPUFO TYP_TEMPUFO
#define ClaSTRING TYP_STRING
#define ClaCSTRING TYP_CSTRING
#define ClaPSTRING TYP_PSTRING
#define ClaMEMO TYP_MEMO
#define ClaGROUP TYP_GROUP
#define ClaCLASS TYP_CLASS
#define ClaARRAY TYP_DIM
#define ClaFIELD TYP_FIELD
#define ClaQUEUE TYP_QUEUE
#define ClaTEMPSTR TYP_TEMPSTR
#define ClaSTACKSTR TYP_STACKSTR
#define ClaPICTURE TYP_PICTURE
#define ClaBLOB TYP_TEMPSTR
#define ClaREF TYP_REFERENCE
#define ClaFILEREF TYP_FILEREF
#define ClaKEYREF TYP_KEYREF
#define ClaQUEUEREF TYP_QUEUEREF
#define ClaCLASSREF TYP_CLASSREF
#define ClaWINDOWREF TYP_WINDOWREF

#endif // _CLACOMP_INCL
