#ifndef _CLAEVAL_
#define _CLAEVAL_

#if _DESIGNER_
#include "claufo.hpp"
#endif



/*-----------------05-06-94 09:38am-----------------
 known_true      :- Evaluate will yield true
 known_false     :- Evaluate will yield false
 known_value     :- Evaluate will yield numeric value
 known_unknown   :- Evaluate will yield unknown
 --------------------------------------------------*/
enum known_state { known_true, known_false, known_unknown };

enum _node_operator {
                     no_select, no_select2,
                     no_power, no_mul, no_div, no_modulus,
                     no_negate,
                     no_add, no_sub,
                     no_concat,
                     no_eq, no_neq, no_lt, no_gt,
                     no_leq, no_geq, no_not,
                     no_and, no_or, no_xor, no_comma,no_colon };
typedef unsigned char node_operator;

enum func_type { ft_user, ft_memory, ft_sub, ft_instring, ft_clip, ft_all,
  ft_center, ft_chr, ft_deformat, ft_format, ft_left, ft_len, ft_lower,
  ft_numeric, ft_right, ft_upper, ft_val, ft_abs, ft_inrange, ft_int, ft_loge,
  ft_log10, ft_random, ft_round, ft_sqrt, ft_sin, ft_cos, ft_tan, ft_asin,
  ft_acos, ft_atan, ft_band, ft_bor, ft_bxor, ft_bshift, ft_today, ft_clock,
  ft_date, ft_day, ft_month, ft_year,ft_age, ft_choose, ft_inlist, ft_getini,

  /*** Generator ***/
  ft_instance = 100, ft_items, ft_extract, ft_separator,
  ft_exists, ft_fileexists, ft_replace, ft_quote, ft_mangle };

enum _node_kind { nk_const, nk_ufo, nk_externufo, nk_unaryop, nk_binaryop,
                  nk_triop, nk_func, nk_missing };

typedef unsigned char node_kind;
enum _node_type { nt_boolean, nt_long, nt_real, nt_picture, nt_string, nt_none };
typedef unsigned char node_type;

#if _DESIGNER_
typedef class HashLink * LinkPtr;
#endif

class constant;
class node
{
  public:
    node_kind kind;
    node_type type;
    unsigned  known; // Level setting at which this definately known
    node ( node_kind k, node_type t );
    node() {};
    char    B();
    node *  make_predicate();
    constant *  make_constant();
    UFO  make_ufo();
    virtual ~node() {};
    virtual void print(void) {};
    virtual void printsql(void) {};
    virtual void printsqlorder(void) {};
    virtual void S() {};
    virtual long L() { return 0;};
    virtual double R() { return 0;};
    virtual known_state An(unsigned,char);
    virtual known_state AnR(unsigned x,char i) { return An(x,i); };
    virtual char AnROK() { return 0; }
    virtual node *Fold(char ) { return this; }; // Once folded not undecided
    virtual void AndSpot(unsigned);
    virtual void FindMin( unsigned ) { return; };
#if _DESIGNER_
    virtual unsigned space_reqd();
    virtual unsigned tomem(vmem * s);
    virtual unsigned frommem(vmem *s);
#endif
};

class constant : public node
{
  public:
    union
    {
      char * string;
      long   integer;
      double real;
    };
    constant( node_kind k, node_type t ) : node(k,t) {}
    constant() {};
    virtual ~constant();
    virtual void print(void);
    virtual void printsql(void);
    virtual void printsqlorder(void);
    virtual void S();
    virtual long L();
    virtual double R();
#if _DESIGNER_
    virtual unsigned space_reqd();
    virtual unsigned tomem(vmem * s);
    virtual unsigned frommem(vmem *s);
#endif
};

class variable : public node
{
  public:
#if _DESIGNER_
    Ufo *var;
    LinkPtr from; // Used by print stuff and gavins stuff
#else
    UFO var;
    char *from; // Used by print stuff and gavins stuff
#endif
    variable( node_kind k, node_type t );
    variable() {};
    virtual ~variable();
    virtual void print(void);
    virtual void printsql(void);
    virtual void printsqlorder(void);
    virtual void S();
    virtual long L();
    virtual double R();
    virtual known_state An(unsigned,char);
    virtual char AnROK();
    virtual node *Fold(char i);
#if _DESIGNER_
    virtual unsigned space_reqd();
    virtual unsigned tomem(vmem * s);
    virtual unsigned frommem(vmem *s);
#endif
};

class operation : public node
{
  public:
    node_operator op;
    node *left,*right;
    operation( node_kind k, node_type t );
    operation() {};
    virtual ~operation();
    virtual void print(void);
    virtual void printsql(void);
    virtual void printsqlorder(void);
    virtual void S();
    virtual long L();
    virtual double R();
    virtual known_state An(unsigned,char);
    virtual known_state AnR(unsigned,char);
    virtual void AndSpot(unsigned);
    virtual node *Fold(char i);
    virtual void FindMin( unsigned );
#if _DESIGNER_
    virtual unsigned space_reqd();
    virtual unsigned tomem(vmem * s);
    virtual unsigned frommem(vmem *s);
#endif
};

class triop : public operation
{
  public:
    node *far_right;
    triop( node_kind k, node_type t );
    triop() {};
    virtual ~triop();
    virtual void print(void);
    virtual void printsql(void);
    virtual void printsqlorder(void);
    virtual void S();
    virtual long L();
    virtual double R();
    virtual known_state An(unsigned,char);
    virtual node *Fold(char i);
#if _DESIGNER_
    virtual unsigned space_reqd();
    virtual unsigned tomem(vmem * s);
    virtual unsigned frommem(vmem *s);
#endif
};

typedef void (* afunc)();
typedef double (* afuncr)();
typedef long (* afuncl)();

struct func_info { int nparams; func_type func; afunc to_call; };

class function_call : public node
{
  public:
    func_type  func;
    char  nparams;   // Restrict to 128 params ...
    node  **params;  // Will array allocated
    afunc to_call;
    char *from;      // Name of function for dumping etc
    function_call( node_kind k, node_type t ) : node(k,t) {}
    function_call() {};
    virtual ~function_call();
    virtual void print(void);
    virtual void printsql(void);
    virtual void printsqlorder(void);
    virtual void S();
    virtual long L();
    virtual double R();
    virtual known_state An(unsigned,char);
    virtual char AnROK();
    virtual node *Fold(char i);
#if _DESIGNER_
    virtual unsigned space_reqd();
    virtual unsigned tomem(vmem * s);
    virtual unsigned frommem(vmem *s);
    char     checkargs();
#endif
};

/*-----------------11-11-94 08:03am-----------------
 Note: I think I need to pass UFOs in as 'field identifiers' rather
 that just addresses. This is to allow you to deal with 'over' cases.
 Consider :-
 MyField  string(8)
 MyReal   real,over(MyField)
 --------------------------------------------------*/
class view_context
{
  public:
    virtual unsigned in_buffer( UFO, char caseless ); // != 0 if you have it
    virtual void relation(node *, node *, node_operator) {};
    virtual void condition(node *) {};
};

class sql_print_context : public view_context
{
  public:
    virtual char * yield_name( UFO ) = 0;
    virtual char * yield_name( char *, char *order, char * ) = 0;
};

class bnode;
class scoperec;

class scoperec
{
  public:
    bnode *symbols;
    scoperec *next;
#pragma save, warn(wall=>off)
    scoperec(bnode* s, scoperec* o) : symbols(s), next(o) {};
#pragma restore
};

extern bnode *symbols;
extern scoperec *scopestack;

#if !_DESIGNER_
extern "C"
{
#pragma save,call(seg_name=>clareng)
#pragma name(prefix=>"")
#endif

void        Cla$PushBind();
void        Cla$PopBind();
void        Cla$BindV( UFO );
void        Cla$BindP( void *, unsigned n, unsigned t );
void        Cla$UnbindA();
void        Cla$UnbindV();
void        Cla$Evaluate();     // Takes string on stack & returns same
void        Cla$EvaluateParsed( node * n ); // Pre-parsed string return on stack
long        Cla$EvaluateParsedL( node * n ); // Pre-parsed string to long
double      Cla$EvaluateParsedR( node * n ); // Pre-parsed string to real
void        Cla$EvaluateParsedU( UFO, node * ); // Pre-parsed string to ufo
int         Cla$EvaluateParsedB( node * n ); // Pre-parsed string to IF xxxx THEN value
UFO         Cla$EvaluateVariable( char *s );
void        Cla$FreeParsed( node * );
                                // Mark parts of tree as unsuitable for evaluation

known_state Cla$AnnotateParsed( node *&, view_context *x );
void        Cla$YieldRelations( node *n, view_context *x, unsigned v );
known_state Cla$EvalS( node *, unsigned n);
char        Cla$EvalR( node *, unsigned n);
UFO         Cla$FindMin( node *, unsigned, UFO, char max, char nocase = 0); // Outcoming UFO to be freed
node *      Cla$ParseCString( char * );     // String to parse is C string passed in
node *      Cla$ParseCStringR( char *& );    // String to parse is C string passed in
char *      Cla$PrintSQL( node *n, sql_print_context * x ); // Returns BIG buffer which should be freed
char *      Cla$PrintSQLOrder( node *n, sql_print_context * x ); // Returns buffer which should be freed

#if !_DESIGNER_
#pragma restore
}
#endif

#if _DESIGNER_
typedef class HashLink * LinkPtr;
#pragma save,call(seg_name=>clareng)
unsigned    Cla$DumpedSpace( node * );      // Space needed to dump eval tree
unsigned    Cla$Dump( vmem *, node * );     // Dump eval tree into memory
node *      Cla$UnDump(vmem *);// Ends up pointing after dumped structure
char *      Cla$Print(node * ); // Returns BIG buffer which should be freed
char *      Cla$PrintExpand(node * ); // Returns BIG buffer which should be freed
char *      Cla$EvaluateSingle( node * );   // Used for 'Ufo return'
#pragma restore
Ufo *  Gen$YieldNode(LinkPtr,unsigned n = 0); // Select of Generator M.V.S.
void   Gen$EvalExtract(unsigned = -1);
void   Gen$EvalReplace(unsigned = -1);
long   Gen$EvalSeparator(long);
long   Gen$EvalInList(node *, LinkPtr);
long   Gen$EvalInstance(LinkPtr);
long   Gen$EvalItems(LinkPtr);
long   Gen$EvalExists(char *);
long   Gen$EvalFileExists(void);
void   Gen$EvalQuote(void);
void   Gen$EvalMangle(void);
char *  Gen$Link2Text(LinkPtr);
LinkPtr Gen$Text2Link(const char *);
LinkPtr Gen$AddSymbol(const char *);
void    Gen$RemoveSymbol(LinkPtr);
#endif

#endif
