#ifndef _CLAPIC_
#define _CLAPIC_

#ifdef __cplusplus
extern "C" {
#endif
#pragma save,name(prefix=>"")
/*-----------------14/05/96 11:35-------------------
  Picture type header file, taken from cla.h
  To be shared by compiler and library
--------------------------------------------------*/

typedef struct
{
  unsigned char   c;      // display character at this place
  unsigned char   phold;  // character place holder type
} PicPlaces;

#define PLACECONST      0   // constant char PicPlaces.phold
#define PLACECOMMA      1   // comma/grouping PicPlaces.phold
#define PLACEDECIMAL    2   // decimal point PicPlaces.phold
#define PLACEFILL       3   // fill character PicPlaces.phold
#define PLACEBLANK      4   // blank fill PicPlaces.phold
#define PLACESIGN       5   // sign PicPlaces.phold
#define PLACECURRENCY   6   // currency PicPlaces.phold
#define PLACEMONTH      7   // month (numeric) PicPlaces.phold
#define PLACEDAY        8   // day PicPlaces.phold
#define PLACEYEAR       9   // year PicPlaces.phold
#define PLACEMONTHA     10  // month (alpha) PicPlaces.phold
#define PLACEHOUR       11  // hour PicPlaces.phold
#define PLACEMINUTE     12  // minute PicPlaces.phold
#define PLACESECOND     13  // second PicPlaces.phold
#define PLACEALPHA      14  // alpha PicPlaces.phold
#define PLACEUPPER      15  // upper case PicPlaces.phold
#define PLACELOWER      16  // lower case PicPlaces.phold
#define PLACEANY        17  // any character PicPlaces.phold
#define PLACESTOP       18  // stop format PicPlaces.phold
#define PLACESEP        19  // seperator PicPlaces.phold
#define PLACEAMPM       20
#define PLACESIGNE      21
#define PLACEVDECIMAL   22
#define PLACEKSTOP      23

typedef struct
{
  unsigned    token:3;    // picture token type
  unsigned    dtype:3;    // data type for conversion
  unsigned    negative:1; // negative value set (run)
  unsigned    blnk:1;     // blank switch (run)
  unsigned    width:8;    // width of picture (num. places
#if __32BIT__
  unsigned    prod_ilka:16; // Work around compiler bug
#endif
} PicHead;

typedef struct {
  PicHead     h;
  char        epoch_shift;// When is 'beginning' of time
  PicPlaces   place[255]; // array of width # of PicPlaces
} PicDesc;

#define PTOKN   0   // numeric (@N) PicDesc.token
#define PTOKE   1   // exponent (@E) PicDesc.token
#define PTOKD   2   // date (@D) PicDesc.token
#define PTOKT   3   // time (@T) PicDesc.token
#define PTOKP   4   // pattern (@P) PicDesc.token
#define PTOKS   5   // string (@S) PicDesc.token
#define PTOKK   6   // key entry (@K) PicDesc.token

#define PICSTRING  0   // convert to string - PicDesc.dtype
#define PICLONG    1   // convert to long - PicDesc.dtype
#define PICREAL    2   // convert to real - PicDesc.dtype
#define PICDECIMAL 3   // convert to decimal - PicDesc.dtype

typedef struct {
  int         iplc;       // integer place count
  int         dplc;       // decimal place count
  int         decpos;     // decimal point position
  int         decchar;    // type of decimal point
  int         moc;        // month place count
  int         yrc;        // year place count
  int         timec;      // time place count
  int         comc;       // number of commas     ***** New

  char        exp;        // exponent indicator
  char        expos;      // exponent position
  char        blank;      // blank fill indicator
  int         inspos;
  int         ovrpos;
  int         lastpos;
  int         slots;
} PicCounts;

#define INTL_MAX_LONG_DATE_WIDTH  32
#define INTL_MAX_SHORT_DATE_WIDTH 16
#define INTL_MAX_LONG_TIME_WIDTH  16
#define INTL_MAX_SHORT_TIME_WIDTH 12

unsigned Cla$storepict(const char *ptok,PicDesc *pdesc);
PicHead Cla$checkpict(const char *ptok);

#pragma restore
#ifdef __cplusplus
}
#endif

#endif
