/*-----------------02-20-95 10:10am-----------------
 Header file for New Clarion Q structures :
   The main idea is to implement Qs as expandable arrays rather than doubly
 linked lists. This should greatly improve random-access performance such as
 adding random data in sorted sequence.
   These new Qs should also support more that 64K elements although I aim
 to move that problem down in to wslarray.cpp.
   Note : Some items 'in' this header file are in CLACOMP.HPP as they are
 used to interface to the compiler
 --------------------------------------------------*/
#ifndef _CLAQ_
#define _CLAQ_
#include <claarray.hpp>
#include <clacdesc.hpp>
#include <claufo.hpp>
#define MAX_KEYS 16
/*-----------------02-20-95 10:16am-----------------
   qrecord manages a record or a Q. If we decide to implement clipping
 then that would go in here.
 There are going to be a lot of these Q records so try not to put any data in
 them ...
 --------------------------------------------------*/
class qrecord : public csl_selement
{
  public:
    void *data;
    qrecord(void *,unsigned);
    virtual ~qrecord();
    void place(void *, unsigned);             // Move data to given area
    void acquire(void *, unsigned); // receive data from given location
};

/*-----------------02-20-95 10:36am-----------------
 I had a bit of a wrestle with this one. I came to the conclusion that I
 wanted Qs to 'look' as clean as possible so I am hiding the classes they
 are built on.
 These queues should take a factor of N out of most previous functions so
 I don't mind k being fractionally bigger.
 ** STOP PRESS: I cannot have these private or I cannot access the functions myself ...
 --------------------------------------------------*/
class qgroup : public csl_exp_array, public ClassDesc
{
  private:
    unsigned size;
    long current_element;
    long pntr;
    void *buffer;
    record_ufos *buffer_ufos, *current_ufos;
    byte sort_direction[MAX_KEYS];
    unsigned sort_sequence[MAX_KEYS];
    byte nkeys;
#pragma save,call(near_call=>on),call(reg_param=>(bx,es,ax,cx,dx,si,di))
    int compare();
    enum find_mode{ find_any, find_after, find_first };
    BOOL find_element(find_mode mode); // 1 for exact match
    BOOL find_element(find_mode mode, long low, long high);
    void sort_insert(long low, long high);
    void sort_merge(long low, long high);
#pragma restore
    unsigned long osequence;
  public:
    qgroup(void *type_cb, void *record_b, unsigned bsize);
    ~qgroup();
    long pointer() { return pntr; };
  // Last add/put or get. Snap-shotted by C layer to give POINTER function
  // Clarion level functions - update q buffer when appropriate
    void ss_clear();              // ss_ routines build up keyed search sequence
    void ss_add_field( int l );   // -ve to indicate descending.
    void ss_add_name(char *c);
    void add(long n);
    void add();                   // add using extent search sequence
    void deleet(long);
    void get(long n);
    void get();
    void put(long n);         // n is CURRENT element
    void sort();
    long records();
  // Library level access functions
    unsigned fields();
    BOOL get_quiet(long n);   // Change internal current element: TRUE in error
    Ufo * field_ufo(unsigned n);        // Ufo for current element - DO NOT DELETE
    Ufo * field_buffer_ufo(unsigned n); // Ufo for Q buffer - DO NOT DELETE
    Ufo * old_field_ufo(unsigned n);        // Knobbles groups from numbering
    Ufo * old_field_buffer_ufo(unsigned n);
    unsigned long sequence;
    int  modified();
};

#endif
