#ifndef _ClaUfo_
#define _ClaUfo_
/*-----------------11-24-93 09:58am-----------------
 This is a class interface for the UFO stuff.
 It should be extensible into a general 'widget' handling mechanism.
 --------------------------------------------------*/
#include "clastst.h"
#include "clwclib.h"

class ClassDesc;
class record_ufos;

/*-----------------11-24-93 10:20am-----------------
 The Ufo class can hopefully be the 'official' interface to the UFO stuff,
 it is thus designed to be l.m.h. and have the same layout as the
 cddw compiler structures.
 --------------------------------------------------*/
#define VARIANT (0)
#define VALUE   (1)
#define COPY    (2)

#define CopyMask  0x1
#define KillNode  0x2
#define NewFormat 0x8 // Only used by Gavin in generator

#ifdef _NO_ASSIGN_
#pragma call(gen_copy_ext=>on,gen_assign_ext=>on)
#endif

struct field_descriptor;

struct VarUfo;

class DynStr;
class vmem
{
  public:
    void         put_unsigned(unsigned);
    unsigned     get_unsigned();
    unsigned     get_string(char * &);
    unsigned     get_string(DynStr &);
    unsigned     put_string(const char *);
    virtual void put_chars(const void *a,unsigned l); // 'Obvious' version extant in claufo
    virtual void get_chars(void *a,unsigned l);
    virtual void put_char(char c) = 0;
    virtual char get_char() = 0;
    virtual void unget_char() = 0;
};

struct Ufo
{
  protected:
    friend class PictureNess; // Pictureness is really a virtual base class
    char type;
    char flags;
  protected:
    Ufo(void) {};
  public:
    void error( int i ) const;
    ~Ufo()
    {
      Free();
    }
    void Assign(const char * s)
    {
      Cla$PushCString(s);
      Pop();
    }
    void String(char * s, unsigned max) const
    {
      Push();
      Cla$PopCString(s, max);
    }
    int operator==(const Ufo &m) const
    {
      return Distinct(m) == 0;
    }
    int operator!=(const Ufo &m) const
    {
      return Distinct(m) != 0;
    }
    int Distinct( double s ) const
    {
      return Distincts(s);
    }
    int Distinct( long l ) const
    {
      return Distincts(l);
    }
    int Distinct( const Ufo &u ) const
    {
      return Distincts(u);
    }
    Ufo *Clone(char value) const;
    unsigned Type() const { return type; };
    int      Distincts( double s ) const;
    /*-----------------08/09/95 10:20-------------------
     New virtual interface, this becomes the server routines for the rest
     --------------------------------------------------*/
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
    virtual void ToMem(vmem &) const = 0;
    virtual void FromMem(vmem &) = 0;
    virtual void OldFromMem(vmem &);
    virtual void Pop();
    virtual void Push() const;
    virtual void DPop();
    virtual void DPush() const;
    virtual double Real() const;
    virtual long Long() const;
    virtual void Free() = 0;
    virtual void Clear(char=0) = 0;
    virtual char *Address() const = 0;
    virtual void Assign(long) = 0;
    virtual void Assign(double) = 0;
    virtual void Assign(const Ufo &) = 0;
    virtual VarUfo *AClone(char copy) const;
    virtual Ufo *Select(unsigned t) const;
    virtual Ufo *Select(unsigned t, unsigned u) const;
    virtual Ufo *Designate(unsigned t) const;
    virtual unsigned Max() const;
    virtual unsigned Size() const = 0;
    virtual unsigned BaseType() const = 0;
    virtual int Distincts( const Ufo & ) const;
    virtual int Distincts( long l ) const;
    virtual int Cleared() const;
    virtual int Null() const;
    virtual void OEMANSITRANS();
    virtual void ANSIOEMTRANS();
    virtual void Bind();
    virtual void Add(const Ufo & );
    virtual void Divide(const Ufo & );
#pragma restore
private:
    void operator=(const Ufo &) const;
};

struct VarUfo : public Ufo
{
  friend class StringNess;

  char *address;
protected:
  void Init(char *m, unsigned size, char copy);
public:
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void Free();
  virtual char *Address() const;
  virtual void SetAddress(char *m);
  virtual void ToMem(vmem &) const;
  virtual void FromMem(vmem &);
#pragma restore
};

struct LongUfo : public VarUfo
{
protected:
  LongUfo() {};
public:
  LongUfo( char *mem, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void Pop();
  virtual void Push() const;
  virtual void DPop();
  virtual void DPush() const;
  virtual double Real() const;
  virtual void Assign(double);
  virtual void Assign(const Ufo &);
  virtual int Distincts( long l ) const;
  virtual int Distincts( const Ufo & ) const;
  virtual int Null() const;
  virtual unsigned BaseType() const;
  // The below need re-writing for each type
  virtual long Long() const ;
  virtual void Clear(char=0);
  virtual void Assign(long);
  virtual VarUfo *AClone(char copy) const;
  virtual unsigned Size() const;
#pragma restore
};


struct ByteUfo : public LongUfo
{
protected:
  ByteUfo() {};
public:
  ByteUfo( char *mem, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual long Long() const;
  virtual void Clear(char=0);
  virtual void Assign(long);
  virtual VarUfo *AClone(char copy) const;
  virtual unsigned Size() const;
#pragma restore
};

struct ShortUfo : public LongUfo
{
protected:
  ShortUfo() {};
public:
  ShortUfo( char *mem, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual long Long() const;
  virtual void Clear(char=0);
  virtual void Assign(long);
  virtual VarUfo *AClone(char copy) const;
  virtual unsigned Size() const;
#pragma restore
};

struct UshortUfo : public LongUfo
{
protected:
  UshortUfo() {};
public:
  UshortUfo( char *mem, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual long Long() const;
  virtual void Clear(char=0);
  virtual void Assign(long);
  virtual VarUfo *AClone(char copy) const;
  virtual unsigned Size() const;
#pragma restore
};

struct TimeUfo : public LongUfo
{
protected:
  TimeUfo() {};
public:
  TimeUfo( char *mem, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual long Long() const;
  virtual void Clear(char=0);
  virtual void Assign(long);
  virtual VarUfo *AClone(char copy) const;
  virtual unsigned Size() const;
#pragma restore
};

struct DateUfo : public LongUfo
{
protected:
  DateUfo() {};
public:
  DateUfo( char *mem, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual long Long() const;
  virtual void Clear(char=0);
  virtual void Assign(long);
  virtual VarUfo *AClone(char copy) const;
  virtual unsigned Size() const;
#pragma restore
};

struct DecimalUfo : public VarUfo
{
  char size,prec;
protected:
  DecimalUfo() {};
public:
  DecimalUfo( char *mem, char sze, char pre, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void ToMem(vmem &) const;
  virtual void FromMem(vmem &);
  virtual void Pop();
  virtual void Push() const;
  virtual double Real() const;
  virtual void Assign(double);
  virtual void Assign(const Ufo &);
  virtual int Distincts( const Ufo & ) const;
  virtual int Distincts( long l ) const;
  virtual int Null() const;
  virtual unsigned BaseType() const;
  virtual long Long() const;
  virtual unsigned Size() const;
  virtual void Assign(long);
  virtual void Clear(char=0);
  virtual void DPop();
  virtual void DPush() const;
  virtual VarUfo *AClone(char copy) const;
#pragma restore
};

struct PdecimalUfo : public DecimalUfo
{
protected:
  PdecimalUfo() {};
public:
  PdecimalUfo( char *mem, char sze, char pre, char copy );
  // The below need re-writing for each type
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void DPop();
  virtual void DPush() const;
  virtual VarUfo *AClone(char copy) const;
#pragma restore
};

struct UlongUfo : public DecimalUfo
{
protected:
  UlongUfo() {};
public:
  UlongUfo( char *mem, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual double Real() const;
  virtual void Assign(long);
  virtual long Long() const;
  virtual void Clear(char=0);
  virtual void DPop();
  virtual void DPush() const;
  virtual VarUfo *AClone(char copy) const;
#pragma restore
};

struct RealUfo : public VarUfo
{
protected:
  RealUfo() {};
public:
  RealUfo( char *mem, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void Pop();
  virtual void Push() const;
  virtual void DPop();
  virtual void DPush() const;
  virtual void Assign(const Ufo &);
  virtual int Distincts( const Ufo & ) const;
  virtual int Distincts( long l ) const;
  virtual int Null() const;
  virtual unsigned BaseType() const;
  virtual long Long() const;
  virtual void Assign(long);
  // The below need re-writing for each type
  virtual double Real() const;
  virtual void Assign(double);
  virtual void Clear(char=0);
  virtual VarUfo *AClone(char copy) const;
  virtual unsigned Size() const;
#pragma restore
};

struct FloatUfo : public RealUfo
{
protected:
  FloatUfo() {};
public:
  FloatUfo( char *mem, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual double Real() const;
  virtual void Assign(double);
  virtual void Clear(char=0);
  virtual VarUfo *AClone(char copy) const;
  virtual unsigned Size() const;
#pragma restore
};

struct Bfloat8Ufo : public RealUfo
{
protected:
  Bfloat8Ufo() {};
public:
  Bfloat8Ufo( char *mem, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual double Real() const;
  virtual void Assign(double);
  virtual void Clear(char=0);
  virtual VarUfo *AClone(char copy) const;
  virtual unsigned Size() const;
#pragma restore
};

struct Bfloat4Ufo : public RealUfo
{
protected:
  Bfloat4Ufo() {};
public:
  Bfloat4Ufo( char *mem, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual double Real() const;
  virtual void Assign(double);
  virtual void Clear(char=0);
  virtual VarUfo *AClone(char copy) const;
  virtual unsigned Size() const;
#pragma restore
};

struct StringNess
{
  unsigned size;
  Ufo *Select( const VarUfo &u, unsigned low, unsigned high ) const;
  void ToMem( const VarUfo &, vmem & ) const;
  void FromMem( VarUfo &, vmem & );
};

struct StringUfo : public VarUfo, public StringNess
{
protected:
  StringUfo() {};
public:
  StringUfo( char *mem, unsigned sze, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void ToMem(vmem &) const;
  virtual void FromMem(vmem &);
  virtual void DPop();
  virtual void DPush() const;
  virtual void Assign(const Ufo &);
  virtual int Distincts( const Ufo & ) const;
  virtual int Distincts( long l ) const;
  virtual int Null() const;
  virtual unsigned BaseType() const;
  virtual long Long() const;
  virtual void Assign(long);
  virtual double Real() const;
  virtual void Assign(double);
  virtual Ufo *Select( unsigned low, unsigned high ) const;
  // The below need re-writing for each type
  virtual void OEMANSITRANS();
  virtual void ANSIOEMTRANS();
  virtual void Pop();
  virtual unsigned Size() const;
  virtual void Push() const;
  virtual void Clear(char=0);
  virtual VarUfo *AClone(char copy) const;
#pragma restore
};

struct PstringUfo : public StringUfo
{
protected:
  PstringUfo() {};
public:
  PstringUfo( char *mem, unsigned sze, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void OEMANSITRANS();
  virtual void ANSIOEMTRANS();
  virtual void Pop();
  virtual void Push() const;
  virtual void Clear(char=0);
  virtual VarUfo *AClone(char copy) const;
#pragma restore
};

struct RefUfo : public StringUfo
{
protected:
  RefUfo() {};
public:
  RefUfo(char *m,unsigned size,char copy);
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void Clear(char=0);
  virtual VarUfo *AClone(char copy) const;
#pragma restore
};

struct CstringUfo : public StringUfo
{
protected:
  CstringUfo() {};
public:
  CstringUfo( char *mem, unsigned sze, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void Pop();
  virtual void Push() const;
  virtual void DPush() const;
  virtual double Real() const;
  virtual void Clear(char=0);
  virtual VarUfo *AClone(char copy) const;
  virtual void OEMANSITRANS();
  virtual void ANSIOEMTRANS();
#pragma restore
};

struct PictureNess : public StringNess
{
  char *picture;
protected:
  PictureNess() {};
public:
  PictureNess( Ufo &, const char *p );
  void ToMem( const VarUfo &, vmem & ) const;
  void FromMem( VarUfo &, vmem & );
  void OldFromMem( VarUfo &, vmem &);
  void Free();
};

struct PictureUfoL : public LongUfo, public PictureNess
{
protected:
  PictureUfoL() {};
public:
  PictureUfoL( char *mem, const char *pic, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void Free();
  virtual long Long() const;
  virtual void Assign(long);
  virtual void Pop();
  virtual void Push() const;
  virtual void DPop();
  virtual unsigned Size() const;
  virtual unsigned BaseType() const;
  virtual void Clear(char=0);
  virtual VarUfo *AClone(char copy) const;
  virtual void ToMem(vmem &) const;
  virtual void FromMem(vmem &);
  virtual void OldFromMem(vmem &);
#pragma restore
};

struct PictureUfoD : public DecimalUfo, public PictureNess
{
protected:
  PictureUfoD() {};
public:
  PictureUfoD( char *mem, const char *pic, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void Free();
  virtual void DPush() const;
  virtual void DPop();
  virtual void Pop();
  virtual void Push() const;
  virtual unsigned Size() const;
  virtual unsigned BaseType() const;
  virtual void Clear(char=0);
  virtual VarUfo *AClone(char copy) const;
  virtual void ToMem(vmem &) const;
  virtual void FromMem(vmem &);
  virtual void OldFromMem(vmem &);
#pragma restore
};

struct PictureUfoR : public RealUfo, public PictureNess
{
protected:
  PictureUfoR() {};
public:
  PictureUfoR( char *mem, const char *pic, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void Free();
  virtual void Pop();
  virtual void Push() const;
  virtual double Real() const;
  virtual void Assign(double);
  virtual unsigned Size() const;
  virtual unsigned BaseType() const;
  virtual void Clear(char=0);
  virtual VarUfo *AClone(char copy) const;
  virtual void ToMem(vmem &) const;
  virtual void FromMem(vmem &);
  virtual void OldFromMem(vmem &);
#pragma restore
};

struct PictureUfoS : public PictureUfoR
{
protected:
  PictureUfoS() {};
public:
  PictureUfoS( char *mem, const char *pic, char copy );
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void Assign(const Ufo &u);
  virtual void Assign(double);
  virtual void Pop();
  virtual void Push() const;
  virtual VarUfo *AClone(char copy) const;
  virtual void ToMem(vmem &) const;
  virtual void FromMem(vmem &);
#pragma restore
};

#define DecSize 32
#define DecPrec 31
struct ValueUfo : public Ufo
{
  union
  {
    long    l;
    double  r;
    StrDesc s;
    char *  d;
  };
  ValueUfo(long l);
  ValueUfo(double d);
  ValueUfo();
  ValueUfo(const char *);
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void ToMem(vmem &) const;
  virtual void FromMem(vmem &);
  virtual void Pop() ;
  virtual void Push() const ;
  virtual void DPop() ;
  virtual void DPush() const ;
  virtual double Real() const ;
  virtual long Long() const ;
  virtual void Free() ;
  virtual void Clear(char=0) ;
  virtual char *Address() const ;
  virtual void Assign(long) ;
  virtual void Assign(double) ;
  virtual void Assign(const Ufo &) ;
  virtual unsigned Size() const ;
  virtual unsigned BaseType() const ;
#pragma restore
};

struct StrValueUfo : public ValueUfo
{
    StrValueUfo();
    StrValueUfo(const char *);
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
    virtual void DPop() ;
    virtual void Free() ;
    virtual void Clear(char=0) ;
    virtual void Assign(long) ;
    virtual void Assign(double) ;
    virtual void Assign(const Ufo &) ;
#pragma restore
};

struct GroupUfo : public StringUfo
{
  ClassDesc *cd;
  record_ufos *ru;
  GroupUfo(char *mem, char *td, char copy);
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual unsigned Max() const;
  virtual Ufo *Select(unsigned n) const;
  virtual Ufo *Designate(unsigned n) const;
  virtual void Free();
  virtual void Clear(char way=0);
  virtual void OEMANSITRANS();
  virtual void ANSIOEMTRANS();
  virtual void Bind();
  virtual int  Cleared() const;
#pragma restore
};

struct ArrayUfo : public VarUfo
{
  VarUfo *child;
  unsigned number;
  void replicate();
  ArrayUfo(VarUfo *ch,unsigned dim,char copy);
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
  virtual void SetAddress(char *m);
  virtual void Free();
  virtual void Clear(char=0);
  virtual int Cleared() const;
  virtual void Assign(long);
  virtual void Assign(double);
  virtual void Assign(const Ufo &);
  virtual VarUfo *AClone(char copy) const;
  virtual Ufo *Select(unsigned t) const;
  virtual unsigned Max() const;
  virtual unsigned Size() const;
  virtual unsigned BaseType() const;
  virtual Ufo *Designate(unsigned n) const;
#pragma restore
};

// Avoid including claeval as claufo is pervasive
class node;

struct ComputedUfo : public Ufo
{
  public:
    ComputedUfo(node *); // Takes possesion, IOW will delete upon destruction
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))
    virtual void Push() const;
    virtual void DPush() const;
    virtual double Real() const;
    virtual long Long() const;
    virtual void Free();
    virtual unsigned Size() const;
    virtual unsigned BaseType() const;
    virtual VarUfo *AClone(char copy) const;
    virtual void ToMem(vmem &) const;
    virtual void FromMem(vmem &);
    virtual void Clear(char=0);
    virtual void Assign(long);
    virtual void Assign(double);
    virtual void Assign(const Ufo &);
    virtual char *Address() const;
#pragma restore
  private:
    node *tree;
};


#ifdef _NO_ASSIGN_
#pragma call(gen_copy_ext=>off, gen_assign_ext=>off)
#endif
#endif
