/* Release 4.00 */
/*-------------------------------------------------------------------------*
*                                                                          *
*  CLWSTG.H - Windows Storage manager                                      *
*                                                                          *
*  COPYRIGHT (C) 1989..1993 Clarion Software Corporation.                  *
*  All Rights Reserved                                                     *
*                                                                          *
*--------------------------------------------------------------------------*/

#ifndef _CLWSTG_INC
#define _CLWSTG_INC

#pragma save
#pragma name(prefix=>"")

#ifdef __cplusplus
extern "Modula2" {
#endif


typedef unsigned SPool;                 // base seg for short pointer
typedef unsigned SHORTADDR;             // offset for short pointer

#define SNIL 0                          // byte at 0 defined to be 0

void      ClwStg$SCreate( SPool *p, unsigned char random, unsigned id);
void      ClwStg$SRemove( SPool *p);
SPool     ClwStg$SCopy( SPool p);

SHORTADDR ClwStg$SNew ( SPool p, unsigned size);  // Zero filled

// NB SMark and SRelease cannot be used on random pools
SHORTADDR ClwStg$SMark( SPool p );
void      ClwStg$SRelease( SPool p, SHORTADDR mark );

// NB SDispose can only used on random pools
void      ClwStg$SDispose( SPool p, SHORTADDR s, unsigned size );

// Asciiz string allocation
SHORTADDR ClwStg$SNewS ( SPool p, char *str);
void      ClwStg$SDisposeS( SPool p, SHORTADDR s);
void      ClwStg$SReplaceS( SPool p, SHORTADDR *s,char *r);

unsigned ClwStg$SLimit(SPool p);  // minimum size of pool
                                  // (also 'slims' pool!)
unsigned ClwStg$SUsed(SPool p);   // for debug only

unsigned ClwStg$SSetId(SPool,unsigned);  // sets user id word
                                         // returns old value


// Segment allocation (Use malloc in preference to these)

unsigned  ClwStg$AllocSeg( unsigned size ); // Does NOT Zero Fill
void      ClwStg$ReallocSeg( unsigned seg, unsigned newsize );
void      ClwStg$FreeSeg( unsigned seg);
unsigned  ClwStg$SizeSeg( unsigned seg);

// Modula-2 style allocation function (no zero fill)
void      ClwStg$ALLOCATE(void **A,   unsigned size);
void      ClwStg$DEALLOCATE(void **A, unsigned size);

// Init function called by iniclib.cpp
void      ClwStg$Init(void);

extern SHORTADDR ClwStg@SPBase;

typedef void (* ClwStg@ErrProc)(unsigned char);
extern  ClwStg@ErrProc ClwStg@Error;


//---------------------------------------------------------

typedef unsigned HashTable;                 // hash table handle


void     ClwStg$HCreate (HashTable *ht, unsigned keysize, unsigned elemsize, unsigned id);
// keysize of 0 means asciiz key

int      ClwStg$HAdd    (HashTable ht, const void *key,void **retelemp);
void    *ClwStg$HFind   (HashTable ht, const void *key);
void    *ClwStg$HKey    (HashTable ht, const void *elemp);
void    *ClwStg$HNext   (HashTable ht, const void *elemp);
unsigned ClwStg$HCount  (HashTable ht);
unsigned ClwStg$HLinkCount (HashTable ht, const void *elemp);
void     ClwStg$HLink   (HashTable ht, const void *elemp);
unsigned ClwStg$HDataSeg(HashTable ht);
unsigned ClwStg$HKeySeg (HashTable ht);
void     ClwStg$HSetKeySeg(HashTable ht, SPool sp); /* Asl use only */
void     ClwStg$HDelete (HashTable ht, const void *elemp);
void     ClwStg$HRemove (HashTable *ht);

// Windows style atom table - i.e. no associated data
typedef unsigned AtomTable;                 // atom table handle
typedef unsigned _ATOM;                      // atom value


AtomTable ClwStg$NewAtomTable  (unsigned id);
_ATOM     ClwStg$AddAtom      (AtomTable at, const char *name);
void      ClwStg$DeleteAtom    (AtomTable at, _ATOM a);
_ATOM     ClwStg$FindAtom     (AtomTable at, const char *name);
_ATOM     ClwStg$LinkAtom     (AtomTable at, _ATOM a);
unsigned  ClwStg$GetAtomName   (AtomTable at, _ATOM a, char *name, unsigned max);
void      ClwStg$DisposeAtomTable (AtomTable at);
char     *ClwStg$GetAtomNamePtr   (AtomTable at, _ATOM a);

#ifdef __cplusplus
}
#endif

#pragma restore

#ifdef __cplusplus
extern "C" {
#endif


#ifdef _INC_WINDOWS
  #define OSHMEM HGLOBAL
#else
  #define OSHMEM unsigned
#endif

void *OSmalloc(unsigned size,OSHMEM *handle);
void *OScalloc(unsigned size,OSHMEM *handle);
void OSrealloc(OSHMEM handle,unsigned newsize);
void OSfree(OSHMEM handle);
void *OSpointer(OSHMEM handle);
void *OShalloc(unsigned long size,OSHMEM *handle);
void OShfree(OSHMEM handle);


// Local routines - Only use if really necessary!!
// First parameter is hinstance, rest as windows routines

unsigned LocalAllocDS(unsigned,unsigned, unsigned);
unsigned LocalReAllocDS(unsigned,unsigned, unsigned, unsigned);
unsigned LocalFreeDS(unsigned,unsigned);
void *   LocalLockDS(unsigned,unsigned);
int      LocalUnlockDS(unsigned,unsigned);
unsigned LocalSizeDS(unsigned,unsigned);
unsigned LocalHandleDS(unsigned,unsigned);


#ifdef __cplusplus
}
#endif

#endif  // CLWSTG_INCL

