#ifndef WINDIA
#define WINDIA

#include "clacomp.hpp"

#ifdef _ASL
#include <asl.h>
#endif

/*-----------------10-15-92 03:58pm-----------------
 * Header file for Clarion Compiler Control Information Exchange
 * (C) 1992 Clarion TopSpeed
 * --------------------------------------------------*/

extern unsigned dumpbase;

#ifdef __cplusplus

/*-----------------10-15-92 04:01pm-----------------
 * Little attempt is made to use C++ privacy to maximise efficiency while
 * using these structures. Note however that the actual format will probably
 * be volatile as the langauge itself changes, therefore assumptions are
 * made at your own peril.
 * --------------------------------------------------*/



/*-----------------10-15-92 04:11pm-----------------
 * The Dumper class is used to set up a short pool, this will be loaded
 * from disk if the string is supplied (valid filenames, I don't check).
 * The delete will de-allocate but will not save, the save function does
 * this. The alloc function is like malloc but the space is allocated from
 * the short pool. Note that I don't offer a free so don't get too excited
 * with the alloc!
 * Head simply returns a pointer to the 'first' element of the pool. This
 * is because I 'waste' a couple of bytes so that NIL is 0 which improves
 * code performance
 * Special Note : Although at first glance this system appears clean, the
 * way segmented pointers are declared in C the segment has to be static
 * therefore you can only have one Dumper active at a give time.
 * Usage:
 *   Dumper *mydumper = new Dumper; // Create 'empty'
 *   Dumper *mydumper = new Dumper("his.dmp"); // Create & load
 *   delete mydumper; // Close down spool
 * --------------------------------------------------*/

typedef class Dcontrol <dumpbase> * Dcontrolptr;
typedef Dcontrolptr <dumpbase> * Dcontrolptrptr;
#pragma save,call(reg_param=>(bx,es,ax,cx,dx,si,di))

class Dumper
{
  public:
    Dcontrolptr hd;
    Dumper();
    Dumper(char *s);
#ifndef _ASL
    ~Dumper()
    {
      Close();
    }
#endif
    void Close();
    void save(char *s);
    void <dumpbase> * alloc(size_t n);
    Dcontrolptr head();
};



typedef char <dumpbase> * Dchars;

typedef class Dusedesc <dumpbase> * Dusedescptr;
class Dusedesc
{
  public:
    TypeCode  UseType;       // type code for use variable (may have array bit set)
    size_t    UseSize;       // size of (whole) associated use variable
    union
    {
      Dchars    UseString;   // When a use variable is a constant string
      Dchars    UsePic;      // picture for use variable
      unsigned  UseDecimals; // size descriptions for a decimal variable
      unsigned  UseElemSize; // array element size (except for decimals etc)
      Dusedescptr UseLink;   // Linked field for compound type
    };
#ifdef _ASL
    void <dumpbase> *operator new(size_t n);
    void operator delete(void <dumpbase> *,size_t);
#endif
};

class Dfontdesc;
typedef Dfontdesc <dumpbase> * Dfontdescptr;

class Dfontdesc
{
  public:
    Dchars    name;
    unsigned  size;
    long      color;
    int       style;
#ifdef _ASL
    Dfontdesc(); // No constructor for non-ASL?
    ~Dfontdesc();
    void <dumpbase> *operator new(size_t n);
    void operator delete(void <dumpbase> *,size_t);
#endif
};

typedef class Dmsgwidth <dumpbase> * Dmsgwidthptr;
class Dmsgwidth
{
  public:
    Dmsgwidth(); // Only can be safely be constructed (using new) from ASL?
    int width;
    Dmsgwidthptr next;
#ifdef _ASL
    void <dumpbase> *operator new(size_t n);
    void operator delete(void <dumpbase> *,size_t);
#endif
};

/*-----------------10-15-92 04:21pm-----------------
 * The base class for all Clarion Window controls. A dialog is basically
 * a list of these. In this (and unless I state otherwise) a next field
 * (or similar) being NIL is considered adequate indication that such
 * an entity does not exist.
 * --------------------------------------------------*/

#endif // _cplusplus

extern void OutText(char *);
extern void do_control_list(Dcontrolptr c);


enum DtypeTag { DTunknown, DTsstring, DTstring, DTimage, DTregion,
                DTline, DTbox,DTellipse, DTentry,
                DTbutton, DTprompt, DToption, DTcheck,
                DTgroup, DTlist,DTcombo, DTspin,
                DTtext, DTcustom, DTmenu, DTitem,
                DTradio,DTmenubar, DTtoolbar, DTapplication,
                DTdialog, DTreport, DTheader,DTfooter,
                DTbreak, DTform, DTdetail, DTvcr,
                DTdroplist,  // internal use by library
                DTdropcombo, // internal use by library,
                DTprogress,
                DTslider,
                DTsheet,
                DTtab,
             DTerror };

typedef char Dtype;
typedef unsigned ctrlflags;

/* Flag values in Dcontrol.flags */

/* for controls */

  #define DFhscroll   0x0001
  #define DFvscroll   0x0002
  #define DFhvscroll  (DFhscroll | DFvscroll)

  #define DFleft      0x0004
  #define DFright     0x0008
  #define DFdecimal   (DFleft | DFright)
  #define DFcenter    0x0010
  #define DFalign     ( DFcenter | DFdecimal | DFright | DFleft )

  #define DFnone      DFcenter          // Use by the slider control
  #define DFboth      DFdecimal

  #define DFskip      0x0020
  #define DFdisable   0x0040
  #define DFscroll    0x0080

  #define DFround     0x0100
  #define DFboxed     0x0200
  #define DFhide      0x0400
  #define DFimm       0x0800
  #define DFtrn       0x1000
  #define DFfull      0x2000
  #define DFnobar     0x4000  // List boxes & combos
  #define DFmeta      0x8000  // Customs

  #define DFwizard    0x0001  // Only on sheets, sorry about the overloading
  #define DFspread    0x0002

/* for screens/reports */

  #define DFhscroll     0x0001                        // screens only
  #define DFvscroll     0x0002                        // screens only
  #define DFhvscroll    (DFhscroll | DFvscroll)       // screens only
  #define DFcenter      0x0010


  #define DFlandscape 0x0001                          // reports only

  #define DFmillimeters 0x0100                        // reports/screens
  #define DFthous       0x0200                        // reports/screens
  #define DFpoints      (DFmillimeters | DFthous)     // reports/screens
  #define DFunits       (DFmillimeters | DFthous | DFpoints)

/* for bands */

  #define DFalone       0x0001
  #define DFabsolute    0x0002

/* for toolbar/menubar section */

  #define DFnomerge     0x0001

/* for menu items/popups */

  #define DFcheck       0x0001
  #define DFfirst       0x0002
  #define DFlast        0x0004
  #define DFright       0x0008
  #define DFseparate    0x0010
  #define DFdisable     0x0040


/* Flag values in Dsmartuseattr.strflags */

  #define SFsum      0x01
  #define SFave      0x02
  #define SFmax      0x03
  #define SFmin      0x04
  #define SFcnt      0x05
  #define SFpageno   0x06
  #define SFusetype  0x0f  //lowest 4 bits indicate smart use type
  #define SFpage     0x10

/* Flag values in Deditattr.editflags */

  #define EFdefault  0x01
  #define EFreq      0x02
  #define EFins      0x04
  #define EFovr      0x08
  #define EFcap      0x10
  #define EFupr      0x20
  #define EFpas      0x40
  #define EFrdo      0x80

/* Flag values in Dvcrattr.vcrflags */

  #define VFcols     0x01
  #define VFvcr      0x02
  #define VFloc      0x04

/* Flag values for Ddialogctrl.winflags */
  #define WFtoolbox     0x0400                        // screens only
  #define WFmaxbox      0x0800                        // screens only
  #define WFsystem      0x1000                        // screens only
  #define WF3d          0x2000                        // screens only
  #define WFmaximize    0x4000                        // screens only
  #define WFiconize     0x8000                        // screens only

  #define WFauto        0x0002                        // screens only
  #define WFmdi         0x0004                        // screens only
  #define WFmodal       0x0008                        // screens only

  #define WFresize      0x0020                        // screens only
  #define WFdouble      0x0040                        // screens only
  #define WFnoframe     (WFresize | WFdouble)         // screens only
  #define WFframebits   (WFresize | WFdouble | WFnoframe)

  #define WFpattern     0x0080                        // screens only

/* Flags values for Dslideattr.slideflags */
  #define SFvertical    0x0001

/*-----------------02-25-93 09:01am-----------------
 A few 'not a values'
 --------------------------------------------------*/

#define NotAKeycode  0
#ifdef __32BIT__
#define NotAnAtValue 0x80000000
#else
#define NotAnAtValue 0x8000
#endif
#define NotAColor    (-1L)
#define NotAStdid    0

#ifdef __cplusplus

class DMega;
class Dmenubar;
class Dtoolbar;
typedef Dmenubar    <dumpbase> *Dmenubarptr;
typedef Dtoolbar    <dumpbase> *Dtoolbarptr;

#ifndef _ASL // TEMP
#pragma save,call(near_call=>on)
#endif
class Deditattr
{
  protected:
    Deditattr(DMega &);
    Deditattr();
#ifdef _ASL
    void transfer();
#endif
    void print();
  public:
    unsigned char editflags;
};

class Dsmartuseattr
{
  protected:
    Dsmartuseattr(DMega &);
    Dsmartuseattr();
#ifdef _ASL
    ~Dsmartuseattr();
    void transfer();
#endif
    void print();
  public:
    unsigned char smartflags;
    int breakdepth; // Pointer to Dchars in JB case
};

class Dfontattr
{
  protected:
    Dfontattr(DMega &);
    Dfontattr();
#ifdef _ASL
    ~Dfontattr();
    void transfer();
#endif
    void print();
  public:
    Dfontdescptr font;
};
typedef Dfontattr <dumpbase> *Dfontattrptr;

class Dalignattr
{
  protected:
    Dalignattr(DMega &);
    Dalignattr();
#ifdef _ASL
    void transfer();
#endif
    void print();
  public:
    int alignment;
};

typedef Dalignattr <dumpbase> *Dalignattrptr;

typedef class Dalrt <dumpbase> * Dalrtptr;
class Dalrt
{
  public:
    Dalrt(); // Only implemented for ASL?
    int key;
    Dalrtptr next;
#ifdef _ASL
    void <dumpbase> *operator new(size_t n);
    void operator delete(void <dumpbase> *,size_t);
#endif
};

class Dfocusattr
{
  protected:
    Dfocusattr(DMega &);
    Dfocusattr();
#ifdef _ASL
    ~Dfocusattr();
    void transfer();
#endif
    void print();
  public:
    Dchars      msg;
    Dchars      tip;
    Dchars      hlp;
    int         keycode;
    Dalrtptr    alrts;
};
typedef Dfocusattr <dumpbase> *Dfocusattrptr;

class Dhaslist
{
  protected:
    Dhaslist(DMega &);
    Dhaslist();
#ifdef _ASL
    ~Dhaslist();
    void transfer();
#endif
    void print();
  public:
    Dcontrolptr fields;
};

typedef Dhaslist <dumpbase> *Dhaslistptr;

class Diconattr
{
  protected:
    Diconattr(DMega &);
    Diconattr();
#ifdef _ASL
    ~Diconattr();
    void transfer();
#endif
    void print();
  public:
    Dchars  icon;
};

typedef Diconattr <dumpbase> *Diconattrptr;

class Dstdattr
{
  protected:
    Dstdattr(DMega &);
    Dstdattr();
#ifdef _ASL
    ~Dstdattr();
    void transfer();
#endif
    void print();
  public:
    int stdid;
};

typedef Dstdattr <dumpbase> *Dstdattrptr;

class Dtabsattr
{
  protected:
    Dtabsattr(DMega &);
    Dtabsattr();
#ifdef _ASL
    ~Dtabsattr();
    void transfer();
#endif
    void print();
  public:
    Dchars tabs;
};

typedef Dtabsattr <dumpbase> *Dtabsattrptr;

class Dvalueattr
{
  protected:
    Dvalueattr(DMega &);
    Dvalueattr();
#ifdef _ASL
    ~Dvalueattr();
    void transfer();
#endif
    void print();
  public:
    Dchars val;
};

typedef Dvalueattr <dumpbase> *Dvalueattrptr;

class Dcursorattr
{
  protected:
    Dcursorattr(DMega &);
    Dcursorattr();
#ifdef _ASL
    ~Dcursorattr();
    void transfer();
#endif
    void print();
  public:
    Dchars cursor;
};

typedef Dcursorattr <dumpbase> *Dcursorattrptr;

class Dparameter
{
  protected:
    Dparameter(DMega &);
    Dparameter();
#ifdef _ASL
    ~Dparameter();
    void transfer();
#endif
    void print();
  public:
    Dchars parameter;
};

typedef Dparameter <dumpbase> *Dparameterptr;

class Datattr
{
  protected:
    Datattr(DMega &);
    Datattr();
#ifdef _ASL
    void transfer();
#endif
    void print();
  public:
    int      x,y,xl,yl;     // AT attribute values
};

typedef Datattr <dumpbase> *Datattrptr;

#define KEEPPRIOR  0
#define KEEPNEXT   1
#define PAGEBEFORE 2
#define PAGEAFTER  3

class Dkeepable
{
  protected:
    Dkeepable(DMega &);
    Dkeepable();
    void print();
#ifdef _ASL
    void transfer();
#endif
  public:
    int keeps[4];
};

typedef Dkeepable <dumpbase> *Dkeepableptr;

class Dfromattr
{
  protected:
    Dfromattr(DMega &);
    Dfromattr();
#ifdef _ASL
    ~Dfromattr();
    void transfer();
#endif
    void print();
  public:
    Dusedescptr from;
};

typedef Dfromattr <dumpbase> *Dfromattrptr;

class Drangectrl
{
  protected:
    Drangectrl(DMega &);
    Drangectrl();
#ifdef _ASL
    void transfer();
#endif
    void print();
  public:
    double lower,upper;
};

class Dspinctrl : public Drangectrl
{
  protected:
    Dspinctrl(DMega &);
    Dspinctrl();
#ifdef _ASL
    void transfer();
#endif
    void print();
  public:
    double step;
};

typedef Dspinctrl <dumpbase> *Dspinctrlptr;

class Dslideattr
{
  protected:
    Dslideattr(DMega &);
    Dslideattr();
#ifdef _ASL
    void transfer();
#endif
    void print();
  public:
    unsigned char slideflags;
};


class Dlabeled
{
  protected:
    Dlabeled();
    Dlabeled(DMega &);
#ifdef _ASL
    void transfer();
    ~Dlabeled();
#endif
    void print();
  public:
    Dchars label;
};

typedef Dlabeled <dumpbase> * Dlabeledptr;

class Dpaperattr
{
  public:
    char signature;
    Dchars pre;
    unsigned short papersize;
    unsigned short paperwidth;
    unsigned short paperheight;
};

#define PAPER_SIGNATURE '.'      // identifies paper struct

typedef Dpaperattr <dumpbase> * Dpaperattrptr;

class Dreportctrl
{
  protected:
    Dreportctrl(DMega &);
    Dreportctrl();
#ifdef _ASL
    void transfer();
    ~Dreportctrl();
#endif
    void print();
  public:
    Dpaperattrptr paper;
};

typedef Dreportctrl <dumpbase> *Dreportctrlptr;

class Dbreakctrl
{
  protected:
    Dbreakctrl(DMega &);
    Dbreakctrl();
#ifdef _ASL
    void transfer();
    ~Dbreakctrl();
#endif
    void print();
  public:
    Dusedescptr breakvar;
};

typedef Dbreakctrl <dumpbase> *Dbreakctrlptr;

class Dcolorattr
{
  protected:
    Dcolorattr(DMega &);
    Dcolorattr();
#ifdef _ASL
    void transfer();
#endif
    void print();
  public:
    long color;
};

typedef Dcolorattr <dumpbase> *Dcolorattrptr;

class Dfillattr
{
  protected:
    Dfillattr(DMega &);
    Dfillattr();
#ifdef _ASL
    void transfer();
#endif
    void print();
  public:
    long fillcolor;
};

typedef Dfillattr <dumpbase> *Dfillattrptr;

class Ddialogctrl
{
  protected:
    Ddialogctrl(DMega &);
    Ddialogctrl();
#ifdef _ASL
    ~Ddialogctrl();
    void transfer();
#endif
    void print();
  public:
    unsigned     winflags;
    unsigned     palette;
    unsigned     timer;
    Dmenubarptr  menufirst;
    Dtoolbarptr  toolfirst;
};
typedef Ddialogctrl <dumpbase> *Ddialogctrlptr;

class Dstatusattr // This should be part of Ddialogctrl but print has probs
{
  protected:
    Dstatusattr();
    Dstatusattr(DMega &x);
#ifdef _ASL
    ~Dstatusattr();
    void transfer();
#endif
    void print();
  public:
    Dmsgwidthptr msgwidthfirst;
};
typedef Dstatusattr <dumpbase> *Dstatusattrptr;

class Dvcrattr
{
  protected:
    Dvcrattr(DMega &);
    Dvcrattr();
#ifdef _ASL
    ~Dvcrattr();
    void transfer();
#endif
    void print();
  public:
    Dchars   vcrpic;
    unsigned vcrnum;
    unsigned char vcrflags;
};

typedef Dvcrattr <dumpbase> *Dvcrattrptr;

class Dlistctrl
{
  protected:
    Dlistctrl(DMega &);
    Dlistctrl();
#ifdef _ASL
    ~Dlistctrl();
    void transfer();
#endif
    void print();
  public:
    unsigned char drop;
    Dusedescptr mark;
};

typedef Dlistctrl <dumpbase> *Dlistctrlptr;

typedef class Dprop <dumpbase> * Dpropptr;
class Dprop
{
  public:
    Dprop(); // Only can be safely be constructed (using new) from ASL?
    Dchars name;
    Dchars value;
    Dpropptr next;
#ifdef _ASL
    void <dumpbase> *operator new(size_t n);
    void operator delete(void <dumpbase> *,size_t);
    ~Dprop();
#endif
};

typedef class Ddid <dumpbase> * Ddidptr;
class Ddid
{
  public:
    Ddid(); // Only can be safely be constructed (using new) from ASL?
    Dchars name;
    Ddidptr next;
#ifdef _ASL
    void <dumpbase> *operator new(size_t n); // I assume NH will want these
    void operator delete(void <dumpbase> *,size_t);
    ~Ddid();
#endif
};

class Ddragattr
{
  protected:
    Ddragattr(DMega &);
    Ddragattr();
#ifdef _ASL
    ~Ddragattr();
    void transfer();
#endif
    void print();
  public:
    Ddidptr drags;
};

typedef Ddragattr <dumpbase> *Ddragattrptr;


class Dcustattr
{
  protected:
    Dcustattr(DMega &);
    Dcustattr();
#ifdef _ASL
    ~Dcustattr();
    void transfer();
#endif
    void print();
  public:
    Dpropptr   property;
    Dchars     classfname;
    Dchars     classname;
};

typedef Dcustattr <dumpbase> *Dcustattrptr;

class Dvisiblectrl : public Datattr, public Dfocusattr,
                     public Dfontattr, public Dcursorattr
{
  protected:
    Dvisiblectrl();
    Dvisiblectrl(DMega &);
    void transfer();
    void print();
};

#pragma restore


class Dcontrol
{
  public:
    Dcontrol(DMega &);
    Dcontrol();
#ifdef _ASL
    void transfer();
    void operator delete(void <dumpbase> *,size_t);
    int  edit(unsigned,void *);
#endif
    void print();
    void <dumpbase> *operator new(size_t n);
    Dtype         kind;          // Type of control
    Dcontrolptr   next;          // Next
    ctrlflags     flags;
    Ddidptr drops;
};


typedef Dcontrol <dumpbase> *Dcontrolptr;

class Dcontrolfield : public Dcontrol
{
  public:
    Dcontrolfield(DMega &);
    Dcontrolfield();
#ifdef _ASL
    ~Dcontrolfield();
#endif
    void transfer();
    void print(unsigned char smartflags);
    int fieldnum; // field equate label, or 0
    Dchars fieldname; // field equate label ( debug=>full only )
};
typedef Dcontrolfield <dumpbase> *Dcontrolfieldptr;

class Dgraphctrl : public Dcontrolfield, public Datattr, public Dcolorattr
{
  public:
    Dgraphctrl(DMega &);
    Dgraphctrl();
#ifdef _ASL
    void transfer();
#endif
    void print();
};
typedef Dgraphctrl <dumpbase> *Dgraphctrlptr;

class Dsolidctrl : public Dgraphctrl, public Dfillattr
{
  public:
    Dsolidctrl(DMega &);
    Dsolidctrl();
#ifdef _ASL
    void transfer();
#endif
    void print();
};
typedef Dsolidctrl <dumpbase> *Dsolidctrlptr;

class Dgenctrl : public Dcontrolfield
{
  public:
    Dgenctrl();
    Dgenctrl(DMega &);
#ifdef _ASL
    void transfer();
    ~Dgenctrl();
#endif
    void print(unsigned char usename = 0); // Takes statusflags (YUK!)
    Dusedescptr use;
};
typedef Dgenctrl <dumpbase> *Dgenctrlptr;

class Dbandctrl : public Dcontrolfield, public Dhaslist, public Dfontattr,
                  public Datattr
{
  public:
    Dbandctrl();
    Dbandctrl(DMega &);
#ifdef _ASL
    void transfer();
#endif
    void print();
};
typedef Dbandctrl <dumpbase> *Dbandctrlptr;

class Dtoolbar : public Dcontrol, public Dvisiblectrl, public Dhaslist
{
  public:
    Dtoolbar(DMega &);
    Dtoolbar();
#ifdef _ASL
    void transfer();
#endif
    void print();
};
typedef Dtoolbar    <dumpbase> *Dtoolbarptr;

class Dmenubar : public Dcontrol, public Dhaslist
{
  public:
    Dmenubar(DMega &);
    Dmenubar();
#ifdef _ASL
    void transfer();
#endif
    void print();
};
typedef Dmenubar <dumpbase> *Dmenubarptr;

class Dreport : public Dcontrol, public Datattr, public Dhaslist,
                public Dlabeled, public Dfontattr, public Dparameter,
                public Dreportctrl, public Dfromattr
{
  public:
    Dreport();
    Dreport(DMega &);
#ifdef _ASL
    void transfer();
#endif
    void print(char list);
};
typedef Dreport <dumpbase> *Dreportptr;

class Ddialog : public Dreport, public Diconattr, public Dfocusattr,
                public Ddialogctrl, public Dcursorattr, public Dstatusattr
{
  public:
    Ddialog();
    Ddialog(DMega &);
#ifdef _ASL
    void transfer();
#endif
    void print();
};
typedef Ddialog <dumpbase> *Ddialogptr;

typedef Ddialog Dapplication;
typedef Dapplication <dumpbase> *Dapplicationptr;

class Dstring : public Dcontrolfield, public Datattr, public Dalignattr,
                public Dparameter, public Dfontattr, public Dcursorattr
{
  public:
    Dstring(DMega &);
    Dstring();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dstring <dumpbase> *Dstringptr;

class Dsstring : public Dgenctrl, public Datattr, public Dalignattr,
                 public Dparameter, public Dfontattr, public Dsmartuseattr,
                 public Dcursorattr
{
  public:
    Dsstring(DMega &);
    Dsstring();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dsstring <dumpbase> *Dsstringptr;

typedef Dstring     Dprompt;

class Dimage : public Dcontrolfield, public Datattr, public Dparameter,
               public Dcursorattr
{
  public:
    Dimage(DMega &);
    Dimage();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dimage <dumpbase> *Dimageptr;

class Dregion : public Dsolidctrl, public Dcursorattr, public Ddragattr
{
  public:
    Dregion(DMega &);
    Dregion();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dregion <dumpbase> *Dregionptr;

typedef Dgraphctrl  Dline;
typedef Dsolidctrl  Dbox;
typedef Dsolidctrl  Dellipse;

class Dentry : public Dgenctrl, public Dvisiblectrl, public Dparameter,
               public Dalignattr, public Deditattr
{
  public:
    Dentry(DMega &);
    Dentry();
#ifdef _ASL
    void transfer();
#endif
    void print(char);
};

typedef Dentry <dumpbase> *Dentryptr;

class Dradio : public Dcontrolfield, public Dvisiblectrl, public Dparameter,
               public Diconattr, public Dvalueattr
{
  public:
    Dradio(DMega &);
    Dradio();
#ifdef _ASL
    void transfer();
#endif
    void print(char);
};

typedef Dradio <dumpbase> *Dradioptr;

class Dtab : public Dcontrolfield, public Dfocusattr,
             public Deditattr,
             public Dparameter, public Diconattr, public Dhaslist
{
  public:
    Dtab(DMega &);
    Dtab();
#ifdef _ASL
    void transfer();
#endif
    void print(unsigned char);
};

typedef Dtab <dumpbase> *Dtabptr;

class Dprogress : public Dgenctrl, public Datattr,
                  public Dfontattr, public Dcursorattr, public Drangectrl
{
  public:
    Dprogress(DMega &);
    Dprogress();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dprogress <dumpbase> *Dprogressptr;

class Dslider : public Dgenctrl, public Dvisiblectrl,
                public Dspinctrl, public Dslideattr
{
  public:
    Dslider(DMega &);
    Dslider();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dslider <dumpbase> *Dsliderptr;

class Dcheck : public Dgenctrl, public Dvisiblectrl, public Dparameter,
               public Diconattr,public Dalignattr
{
  public:
    Dcheck(DMega &);
    Dcheck();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dcheck <dumpbase> *Dcheckptr;

class Dbutton : public Dradio, public Dstdattr, public Deditattr
{
  public:
    Dbutton(DMega &);
    Dbutton();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dbutton <dumpbase> *Dbuttonptr;

class Doption : public Dgenctrl, public Dvisiblectrl, public Dparameter,
                public Dhaslist
{
  public:
    Doption(DMega &);
    Doption();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Doption <dumpbase> *Doptionptr;
typedef Doption Dsheet;
typedef Dsheet <dumpbase> *Dsheetptr;

class Dgroup : public Dcontrolfield, public Dvisiblectrl, public Dparameter,
               public Dhaslist
{
  public:
    Dgroup(DMega &);
    Dgroup();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dgroup <dumpbase> *Dgroupptr;

class Dspin : public Dentry, public Dspinctrl, public Dfromattr
{
  public:
    Dspin();
    Dspin(DMega &);
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dspin <dumpbase> *Dspinptr;

class Dtext : public Dentry
{
  public:
    Dtext(DMega &);
    Dtext();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dtext <dumpbase> *Dtextptr;

class Dlist : public Dgenctrl, public Dtabsattr, public Dvisiblectrl,
              public Dlistctrl,public Dfromattr, public Dalignattr,
              public Dvcrattr, public Ddragattr
{
  public:
    Dlist();
    Dlist(DMega &);
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dlist <dumpbase> *Dlistptr;

class Dcombo : public Dlist, public Dparameter, public Deditattr
{
  public:
    Dcombo();
    Dcombo(DMega &);
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dcombo <dumpbase> *Dcomboptr;

class Dvcr : public Dcontrolfield, public Dvisiblectrl, public Dvcrattr
{
  public:
    Dvcr();
    Dvcr(DMega &);
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dvcr <dumpbase> *Dvcrptr;

class Dcustom : public Dgenctrl, public Dvisiblectrl, public Dalignattr,
                public Dcustattr, public Dparameter
{
  public:
    Dcustom();
    Dcustom(DMega &);
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dcustom <dumpbase> *Dcustomptr;

class Ditem : public Dgenctrl, public Dparameter, public Dfocusattr,
              public Dstdattr
{
  public:
    Ditem(DMega &);
    Ditem();
#ifdef _ASL
    void transfer();
#endif
    void print(char);
};

typedef Ditem <dumpbase> *Ditemptr;

class Dmenu : public Ditem, public Dhaslist
{
  public:
    Dmenu(DMega &);
    Dmenu();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dmenu <dumpbase> *Dmenuptr;

class Dbreak : public Dcontrol, public Dhaslist, public Dbreakctrl,
               public Dlabeled
{
  public:
    Dbreak(DMega &);
    Dbreak();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dbreak <dumpbase> *Dbreakptr;

class Dform : public Dbandctrl
{
  public:
    Dform(DMega &);
    Dform();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Dform <dumpbase> *Dformptr;

class Ddetail : public Dbandctrl, public Dkeepable, public Dlabeled
{
  public:
    Ddetail(DMega &);
    Ddetail();
#ifdef _ASL
    void transfer();
#endif
    void print();
};

typedef Ddetail <dumpbase> *Ddetailptr;

typedef Ddetail   Dfooter;
typedef Ddetail   Dheader;

class DMega : public Dgenctrl, public Dvisiblectrl,
              public Diconattr, public Dparameter,
              public Dhaslist, public Dalignattr,
              public Dkeepable, public Dtabsattr, public Dspinctrl,
              public Dreportctrl, public Ddialogctrl, public Dcolorattr,
              public Dstdattr, public Dfillattr, public Dlistctrl,
              public Dfromattr, public Dbreakctrl, public Dlabeled,
              public Dstatusattr, public Deditattr, public Dsmartuseattr,
              public Dcustattr, public Dvcrattr, public Dvalueattr,
              public Ddragattr, public Dslideattr

{
  public:
    DMega();
    void operator=(DMega &);
};


typedef Dstring     <dumpbase> *Dstringptr;
typedef Dprompt     <dumpbase> *Dpromptptr;
typedef Dimage      <dumpbase> *Dimageptr;
typedef Dline       <dumpbase> *Dlineptr;
typedef Dbox        <dumpbase> *Dboxptr;
typedef Dellipse    <dumpbase> *Dellipseptr;
typedef Dentry      <dumpbase> *Dentryptr;
typedef Dbutton     <dumpbase> *Dbuttonptr;
typedef Dcheck      <dumpbase> *Dcheckptr;
typedef Dradio      <dumpbase> *Dradioptr;
typedef Doption     <dumpbase> *Doptionptr;
typedef Dgroup      <dumpbase> *Dgroupptr;
typedef Dspin       <dumpbase> *Dspinptr;
typedef Dtext       <dumpbase> *Dtextptr;
typedef Dlist       <dumpbase> *Dlistptr;
typedef Dcombo      <dumpbase> *Dcomboptr;
typedef Ditem       <dumpbase> *Ditemptr;
typedef Dmenu       <dumpbase> *Dmenuptr;
typedef Dtoolbar    <dumpbase> *Dtoolbarptr;
typedef Dmenubar    <dumpbase> *Dmenubarptr;
typedef Dbreak      <dumpbase> *Dbreakptr;
typedef Dform       <dumpbase> *Dformptr;
typedef Dheader     <dumpbase> *Dheaderptr;
typedef Dfooter     <dumpbase> *Dfooterptr;
typedef Dform       <dumpbase> *Dformptr;
typedef Ddetail     <dumpbase> *Ddetailptr;
typedef Dsstring    <dumpbase> *Dsstringptr;
typedef Dsheet      <dumpbase> *Dsheetptr;

#ifndef _ASL // TEMP
#pragma restore
#endif

#endif


#endif // _cplusplus
